from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

from collections import defaultdict
from datetime import datetime, timedelta
import six
import mock
import random
import tempfile

from rhsm import config
from subscription_manager.cert_sorter import CertSorter
from subscription_manager.cache import EntitlementStatusCache, ProductStatusCache, \
        OverrideStatusCache, ProfileManager, InstalledProductsManager, ReleaseStatusCache, \
        PoolStatusCache, ContentAccessModeCache, SupportedResourcesCache, AvailableEntitlementsCache, \
        SyspurposeValidFieldsCache, CurrentOwnerCache
from subscription_manager.facts import Facts
from subscription_manager.lock import ActionLock
from rhsm.certificate import GMT
from subscription_manager.gui.utils import AsyncWidgetUpdater, handle_gui_exception
from rhsm.certificate2 import Version
from subscription_manager.certdirectory import EntitlementDirectory, ProductDirectory

from rhsm.certificate import parse_tags
from rhsm.certificate2 import EntitlementCertificate, ProductCertificate, \
        Product, Content, Order
from rhsm import profile
from rhsm import ourjson as json
from rhsm.certificate2 import CONTENT_ACCESS_CERT_TYPE

# config file is root only, so just fill in a stringbuffer
cfg_buf = """
[foo]
bar =

[server]
hostname = server.example.conf
prefix = /candlepin
port = 8443
insecure = 1
ssl_verify_depth = 3
ca_cert_dir = /etc/rhsm/ca/
proxy_hostname = notaproxy.grimlock.usersys.redhat.com
proxy_port = 4567
proxy_user = proxy_user
proxy_password = proxy_password
no_proxy =

[rhsm]
baseurl = https://content.example.com
repomd_gpg_url =
repo_ca_cert = %(ca_cert_dir)sredhat-uep.pem
productCertDir = /etc/pki/product
entitlementCertDir = /etc/pki/entitlement
consumerCertDir = /etc/pki/consumer
ca_cert_dir = /etc/rhsm/ca/

[rhsmcertd]
certCheckInterval = 240

[logging]
default_log_level = DEBUG
"""


class StubConfig(config.RhsmConfigParser):
    def __init__(self, config_file=None, defaults=None):
        defaults = defaults or config.DEFAULTS
        config.RhsmConfigParser.__init__(self, config_file=config_file, defaults=defaults)
        self.raise_io = None
        self.fileName = config_file
        self.store = defaultdict(dict)

    # instead of reading a file, let's use the stringio
    def read(self, filename):
        self.readfp(six.StringIO(cfg_buf), "foo.conf")

    # this way our test can put some values in and have them used during the run
    def get(self, section, key):
        # print self.sections()
        value = super(StubConfig, self).get(section, key)
        test_value = None
        try:
            test_value = self.store[section][key]
        except KeyError:
            test_value = None

        if test_value:
            return test_value
        else:
            return value

    def set(self, section, key, value):
        # print self.sections()
        self.store[section][key] = value

    def items(self, section):
        # Attempt to return the items from the store for the given section.
        # This allows tests using this stub to set arbitrary keys in a given
        # section and iterate over them with their values.
        items_from_store = self.store[section]
        if len(items_from_store) > 0:
            return list(items_from_store.items())
        return config.RhsmConfigParser.items(self, section)

    def save(self, config_file=None):
        if self.raise_io:
            raise IOError
        return None


def stubInitConfig():
    return StubConfig()


# create a global CFG object,then replace it with our own that candlepin
# read from a stringio
#config.get_config_parser(config_file="test/rhsm.conf")
config.CFG = StubConfig()

# we are not actually reading test/rhsm.conf, it's just a placeholder
config.CFG.read("test/rhsm.conf")


class MockActionLock(ActionLock):
    PATH = tempfile.mkstemp()[1]


class StubProduct(Product):

    def __init__(self, product_id, name=None, version=None,
                 architectures=None, provided_tags=None,
                 os=None, brand_name=None):

        # Initialize some defaults:
        if not name:
            name = product_id

        if not architectures:
            architectures = ["x86_64"]

        if not version:
            version = "1.0"

        # Tests sadly pass these in as a flat string. # TODO
        if provided_tags:
            provided_tags = parse_tags(provided_tags)

        super(StubProduct, self).__init__(id=product_id, name=name, version=version,
                                          architectures=architectures,
                                          provided_tags=provided_tags,
                                          brand_type=os, brand_name=brand_name)


class StubContent(Content):

    def __init__(self, label, name=None, vendor="",
            url="", gpg="", enabled=1, metadata_expire=None, required_tags="",
            content_type="yum"):
        name = label
        if name:
            name = name
        if required_tags:
            required_tags = parse_tags(required_tags)
        super(StubContent, self).__init__(content_type=content_type, name=name, label=label,
                                          vendor=vendor, url=url, gpg=gpg, enabled=enabled,
                                          metadata_expire=metadata_expire,
                                          required_tags=required_tags)


class StubProductCertificate(ProductCertificate):

    def __init__(self, product, provided_products=None, start_date=None,
            end_date=None, provided_tags=None):

        products = [product]
        if provided_products:
            products = products + provided_products

        self.name = product.name

        # product certs are all version 1.0
        version = Version("1.0")

        # TODO: product should be a StubProduct, check for strings coming in and error out
        self.product = product
        self.provided_products = []
        if provided_products:
            self.provided_products = provided_products

        self.provided_tags = set()
        if provided_tags:
            self.provided_tags = set(provided_tags)

        if not start_date:
            start_date = datetime.now() - timedelta(days=100)
        if not end_date:
            end_date = datetime.now() + timedelta(days=365)

        path = "/path/to/fake_product.pem"

        super(StubProductCertificate, self).__init__(products=products,
                                                     serial=random.randint(1, 10000000),
                                                     start=start_date, end=end_date,
                                                     version=version, path=path)

    def __str__(self):
        s = []
        s.append('StubProductCertificate:')
        s.append('===================================')
        for p in self.products:
            s.append(str(p))
        return '\n'.join(s)


class StubEntitlementCertificate(EntitlementCertificate):

    def __init__(self, product, provided_products=None, start_date=None, end_date=None,
            content=None, quantity=1, stacking_id=None, sockets=2, service_level=None,
            ram=None, pool=None, ent_id=None, entitlement_type=None):

        # If we're given strings, create stub products for them:
        if isinstance(product, str):
            product = StubProduct(product)
        if provided_products:
            temp = []
            for p in provided_products:
                temp.append(StubProduct(p))
            provided_products = temp

        products = []
        if product:
            products.append(product)
        if provided_products:
            products = products + provided_products

        if not start_date:
            start_date = datetime.utcnow()
        if not end_date:
            end_date = start_date + timedelta(days=365)

        # to simulate a cert with no product
        sku = None
        name = None
        if product:
            sku = product.id
            name = product.name
        order = Order(name=name, number="592837", sku=sku,
                    stacking_id=stacking_id, socket_limit=sockets,
                    service_level=service_level, quantity_used=quantity,
                    ram_limit=ram)
        order.warning_period = 42

        if content is None:
            content = []

        path = "/tmp/fake_ent_cert.pem"
        self.is_deleted = False

        version = Version("3.0")

        # might as well make this a big num since live serials #'s are already > maxint
        self.serial = random.randint(1000000000000000000, 10000000000000000000000)
        # write these to tmp, could we abuse PATH thing in certs for tests?

        path = "/tmp/fake_ent_cert-%s.pem" % self.serial
        super(StubEntitlementCertificate, self).__init__(path=path, products=products, order=order,
                                                         content=content, pool=pool, start=start_date,
                                                         end=end_date, serial=self.serial, version=version)
        if ent_id:
            self.subject = {'CN': ent_id}

        self._entitlement_type = entitlement_type or 'Basic'

    @property
    def entitlement_type(self):
        return self._entitlement_type

    def delete(self):
        self.is_deleted = True

    def is_expiring(self, on_date=None):
        gmt = datetime.utcnow()
        if on_date:
            gmt = on_date
        gmt = gmt.replace(tzinfo=GMT())
        warning_time = timedelta(days=int(self.order.warning_period))
        return self.valid_range.end() - warning_time < gmt


class StubCertificateDirectory(EntitlementDirectory):
    """
    Stub for mimicing behavior of an on-disk certificate directory.
    Can be used for both entitlement and product directories as needed.
    """

    path = "this/is/a/stub/cert/dir"
    expired = False

    def __init__(self, certificates=None):
        self.certs = certificates
        if certificates is None:
            self.certs = []
        self.list_called = False

    def list(self):
        self.list_called = True
        return self.certs

    def _check_key(self, cert):
        """
        Fake filesystem access here so we don't try to read real keys.
        """
        return True

    def getCerts(self):
        return self.certs


# so we can use a less confusing name when we use this stub
class StubEntitlementDirectory(StubCertificateDirectory):
    path = "this/is/a/stub/ent/cert/dir"

    def list_valid_with_content_access(self):
        return [x for x in self.list_with_content_access() if self._check_key(x) and x.is_valid()]

    def list(self):
        certs = super(StubEntitlementDirectory, self).list()
        return [cert for cert in certs if cert.entitlement_type != CONTENT_ACCESS_CERT_TYPE]

    def list_with_content_access(self):
        return super(StubEntitlementDirectory, self).list()


class StubProductDirectory(StubCertificateDirectory, ProductDirectory):
    """
    Stub for mimicing behavior of an on-disk certificate directory.
    Can be used for both entitlement and product directories as needed.
    """

    path = "this/is/a/stub/product/cert/dir"

    def __init__(self, certificates=None, pids=None):
        """
        Pass list of product ID strings instead of certificates to have
        stub product certs created for you.
        """
        if pids is not None:
            certificates = []
            for pid in pids:
                certificates.append(StubProductCertificate(StubProduct(pid)))
        super(StubProductDirectory, self).__init__(certificates)

    # real version just calls refresh on it's set of ProductDirs, that don't
    # exist here, so this needs to be stubbed.
    def refresh(self):
        pass


class StubConsumerIdentity(object):
    CONSUMER_NAME = "John Q Consumer"
    CONSUMER_ID = "211211381984"
    SERIAL = "23234523452345234523453453434534534"

    def __init__(self, keystring, certstring):
        self.key = keystring
        self.cert = certstring

    @classmethod
    def existsAndValid(cls):
        return True

    @classmethod
    def exists(cls):
        return False

    def getConsumerName(self):
        return StubConsumerIdentity.CONSUMER_NAME

    def getConsumerId(self):
        return StubConsumerIdentity.CONSUMER_ID

    def getSerialNumber(self):
        return StubConsumerIdentity.SERIAL

    @classmethod
    def read(cls):
        return StubConsumerIdentity("", "")

    @classmethod
    def certpath(cls):
        return ""

    @classmethod
    def keypath(cls):
        return ""


class StubUEP(object):
    def __init__(self, host=None, ssl_port=None, handler=None,
                 username=None, password=None,
                 proxy_hostname=None, proxy_port=None,
                 proxy_user=None, proxy_password=None,
                 cert_file=None, key_file=None, restlib_class=None):
        self.registered_consumer_info = {"uuid": 'dummy-consumer-uuid'}
        self.environment_list = []
        self.called_unregister_uuid = None
        self.called_unbind_uuid = None
        self.called_unbind_serial = []
        self.called_unbind_pool_id = []
        self.username = username
        self.password = password
        self._capabilities = []
        self.conn = mock.MagicMock()
        self.conn.ALPHA = 0.9
        self.conn.smoothed_rt = 1.1

    def reset(self):
        self.called_unregister_uuid = None
        self.called_unbind_uuid = None
        self.called_unbind_serial = []
        self.called_unbind_pool_id = []

    def has_capability(self, capability):
        return capability in self._capabilities

    def supports_resource(self, resource):
        return False

    def get_supported_resources(self):
        return []

    def registerConsumer(self, name, type, facts, owner, environment, keys,
                         installed_products, content_tags):
        return self.registered_consumer_info

    def unregisterConsumer(self, uuid):
        self.called_unregister_uuid = uuid

    def getOwnerList(self, username):
        return [{'key': 'dummyowner'}]

    def getOwner(self, consumer_uuid):
        return {'key': 'dummyowner'}

    def updatePackageProfile(self, uuid, pkg_dicts):
        pass

    def getProduct(self):
        return {}

    def getRelease(self, consumerId):
        return {'releaseVer': ''}

    def getServiceLevelList(self, owner):
        return ['Pro', 'Super Pro', 'ProSumer']

    def updateConsumer(self, consumer, facts=None, installed_products=None,
                       guest_uuids=None, service_level=None, release=None, autoheal=None,
                       content_tags=None, addons=None, role=None, usage=None):
        return consumer

    def setEnvironmentList(self, env_list):
        self.environment_list = env_list

    def getEnvironmentList(self, owner):
        return self.environment_list

    def setConsumer(self, consumer):
        self.consumer = consumer

    def getConsumer(self, consumerId, username=None, password=None):
        if hasattr(self, 'consumer') and self.consumer:
            return self.consumer
        if six.callable(self.registered_consumer_info):
            return self.registered_consumer_info()
        return self.registered_consumer_info

    def unbindAll(self, consumer):
        self.called_unbind_uuid = consumer

    def unbindBySerial(self, consumer, serial):
        self.called_unbind_serial.append(serial)

    def unbindByPoolId(self, consumer_uuid, pool_id):
        self.called_unbind_pool_id.append(pool_id)

    def getCertificateSerials(self, consumer):
        return []

    def getCompliance(self, uuid, on_data=None):
        return {}

    def getSyspurposeCompliance(self, uuid, on_date=None):
        return self.syspurpose_compliance_status

    def setSyspurposeCompliance(self, status):
        self.syspurpose_compliance_status = status

    def getEntitlementList(self, uuid):
        return [{'id': 'ent1'}, {'id': 'ent2'}]

    def getPoolsList(self, uuid, listAll, active_on, owner):
        return [{'id': 'pool1'}, {'id': 'pool2'}]

    def getSubscriptionList(self, owner):
        return [{'id': 'sub1'}, {'id': 'sub2'}]

    def getContentOverrides(self, uuid):
        return []

    def getOwnerSyspurposeValidFields(self, owner_key):
        return {
            "owner": {
                "key": "ff80808172dc51a10172dc51cb3e000"
            },
            "systemPurposeAttributes": {
                "addons": [], "usage": [], "support_level": [], "roles": []
            }
        }


class StubBackend(object):
    def __init__(self, uep=None):
        self.cp_provider = StubCPProvider()
        self.entitlement_dir = None
        self.product_dir = None
        self.content_connection = None
        self.cs = StubCertSorter()
        self.overrides = None
        self.certlib = None

    def on_cert_check_timer(self):
        pass

    def update(self):
        pass


class StubContentConnection(object):
    proxy_hostname = None
    proxy_port = None


class StubFacts(Facts):
    def __init__(self, fact_dict=None, facts_changed=True):
        fact_dict = fact_dict or {}
        self.facts = fact_dict

        self.delta_values = {}
        # Simulate the delta as being the new set of facts provided.
        if facts_changed:
            self.delta_values = self.facts

    def get_facts(self, refresh=True):
        return self.facts

    def has_changed(self):
        return self.delta_values

    def update_check(self, uep, consumer_uuid, force=False):
        uep.updateConsumer(consumer_uuid, self.facts)

    def get_last_update(self):
        return None

    def write_cache(self):
        pass

    def delete_cache(self):
        self.server_status = None


class StubConsumer(object):
    def __init__(self):
        self.uuid = None

    def is_valid(self):
        return True

    def reload(self):
        pass

    def getConsumerId(self):
        return "12341234234"


class StubEntActionInvoker(object):
    def __init__(self, uep=None):
        self.uep = uep or StubUEP()

    def update(self):
        pass


class StubCertSorter(CertSorter):

    def __init__(self):
        super(StubCertSorter, self).__init__()

    def update_product_manager(self):
        pass

    def _parse_server_status(self):
        # Override this method to just leave all fields uninitialized so
        # tests can do whatever they wish with them.
        pass


class StubCPProvider(object):

    def __init__(self):
        self.cert_file = StubConsumerIdentity.certpath()
        self.key_file = StubConsumerIdentity.keypath()
        self.consumer_auth_cp = StubUEP()
        self.basic_auth_cp = StubUEP()
        self.no_auth_cp = StubUEP()
        self.content_connection = StubContentConnection()

    def set_connection_info(self,
        host=None,
        ssl_port=None,
        handler=None,
        cert_file=None,
        key_file=None,
        proxy_hostname_arg=None,
        proxy_port_arg=None,
        proxy_user_arg=None,
        proxy_password_arg=None,
        no_proxy_arg=None,
        correlation_id=None,
        restlib_class=None):
        pass

    def set_content_connection_info(self, cdn_hostname=None, cdn_port=None):
        pass

    def set_user_pass(self, username=None, password=None):
        pass

    def set_correlation_id(self, correlation_id):
        pass

    # tries to write to /var/lib and it reads the rpm db
    def clean(self):
        pass

    def get_consumer_auth_cp(self):
        return self.consumer_auth_cp

    def get_basic_auth_cp(self):
        return self.basic_auth_cp

    def get_no_auth_cp(self):
        return self.no_auth_cp

    def get_content_connection(self):
        return self.content_connection


class StubEntitlementStatusCache(EntitlementStatusCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubPoolStatusCache(PoolStatusCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubProductStatusCache(ProductStatusCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubOverrideStatusCache(OverrideStatusCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubReleaseStatusCache(ReleaseStatusCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubAvailableEntitlementsCache(AvailableEntitlementsCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubContentAccessModeCache(ContentAccessModeCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubSupportedResourcesCache(SupportedResourcesCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubSyspurposeValidFieldsCache(SyspurposeValidFieldsCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubCurrentOwnerCache(CurrentOwnerCache):

    def write_cache(self, debug=False):
        pass

    def delete_cache(self):
        self.server_status = None


class StubPool(object):

    def __init__(self, poolid):
        self.id = poolid


class StubAsyncUpdater(AsyncWidgetUpdater):

    def update(self, widget_update, backend_method, args=[], kwargs={}, exception_msg=None, callback=None):
        try:
            result = backend_method(*args, **kwargs)
            if callback:
                callback(result)
        except Exception as e:
            message = exception_msg or str(e)
            handle_gui_exception(e, message, self.parent_window)
        finally:
            widget_update.finished()


class StubInstalledProductsManager(InstalledProductsManager):
    def write_cache(self):
        pass

    def delete_cache(self):
        self.server_status = None


class StubProfileManager(ProfileManager):

    def write_cache(self):
        pass

    def delete_cache(self):
        self.server_status = None

    def _get_current_profile(self):
        mock_packages = [
                  profile.Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                  profile.Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        self._current_profile = StubRpmProfile(mock_packages=mock_packages)
        return self._current_profile

    # NOTE: ProfileManager asks the ether for it's profile, so
    # stub that as well
    def _get_profile(self, profile_type):
        return self._get_current_profile()


# could be a Mock
class StubRpmProfile(profile.RPMProfile):
    def __init__(self, from_file=None, mock_packages=None):
        self.mock_packages = mock_packages
        mock_file = self._mock_pkg_profile_file()

        # create an RPMProfile from the mock_packages list
        super(StubRpmProfile, self).__init__(from_file=mock_file)

    def _get_packages(self):
        """return a list of profile.Package objects"""
        if self.mock_packages:
            return self.mock_packages
        return []

    def _mock_pkg_profile_file(self):
        """
        Turn a list of package objects into an RPMProfile object.
        """

        packages = self._get_packages()
        dict_list = []
        for pkg in packages:
            dict_list.append(pkg.to_dict())

        mock_file = mock.Mock()
        mock_file.read = mock.Mock(return_value=json.dumps(dict_list))

        return mock_file



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import logging

from rhsm.certificate import DateRange
import subscription_manager.injection as inj
from subscription_manager.isodate import parse_date

log = logging.getLogger(__name__)


class ValidProductDateRangeCalculator(object):

    def __init__(self, uep=None):
        uep = uep or inj.require(inj.CP_PROVIDER).get_consumer_auth_cp()
        self.identity = inj.require(inj.IDENTITY)
        if self.identity.is_valid():
            self.prod_status_cache = inj.require(inj.PROD_STATUS_CACHE)
            self.prod_status = self.prod_status_cache.load_status(
                    uep, self.identity.uuid)

    def calculate(self, product_hash):
        """
        Calculate the valid date range for the specified product based on
        today's date.

        Partially entitled products are considered when determining the
        valid range.

        NOTE:
        The returned date range will be in GMT, so keep this in mind when
        presenting these dates to the user.
        """
        # If we're not registered, don't return a valid range:
        if not self.identity.is_valid():
            return None

        if self.prod_status is None:
            return None

        for prod in self.prod_status:
            if product_hash != prod['productId']:
                continue

            # Found the product ID requested:
            if 'startDate' in prod and 'endDate' in prod:

                # Unentitled product:
                if prod['startDate'] is None or prod['endDate'] is None:
                    return None

                return DateRange(parse_date(prod['startDate']),
                    parse_date(prod['endDate']))
            else:
                # If startDate / endDate not supported
                log.warn("Server does not support product date ranges.")
                return None

        # At this point, we haven't found the installed product that was
        # asked for, which could indicate the server somehow doesn't know
        # about it yet. This is extremely weird and should be unlikely,
        # but we will log and handle gracefully:
        log.error("Requested status for installed product server does not "
                "know about: %s" % product_hash)
        return None



from __future__ import print_function, division, absolute_import

import mock

from . import fixture

from subscription_manager import model


def create_mock_content(name=None, url=None, gpg=None, enabled=None, content_type=None, tags=None):
    mock_content = mock.Mock()
    mock_content.name = name or "mock_content"
    mock_content.label = name
    mock_content.url = url or "http://mock.example.com/%s/" % name
    mock_content.gpg = gpg or "path/to/gpg"
    mock_content.enabled = enabled or True
    mock_content.content_type = content_type or "yum"
    mock_content.tags = tags or []
    return mock_content


class TestContent(fixture.SubManFixture):
    def test_init(self):
        content = model.Content("content-label",
                                "content name",
                                "http://contenturl.example.com",
                                "/path/to/gpg",
                                ["product-content-tag"],
                                cert=None)

        self._check_attrs(content)
        self.assertTrue(isinstance(content.tags, list))

    def _check_attrs(self, content):
        attrs = ['content_type', 'name', 'label', 'url', 'gpg', 'tags', 'cert']
        for attr in attrs:
            self.assertTrue(hasattr(content, attr))


class TestEntitlement(fixture.SubManFixture):
    def test_empty_init(self):
        e = model.Entitlement()
        self.assertTrue(hasattr(e, 'contents'))

    def test_init_empty_contents(self):
        e = model.Entitlement(contents=[])
        self.assertTrue(hasattr(e, 'contents'))
        self.assertEqual(e.contents, [])

    def test_contents(self):
        contents = [mock.Mock(), mock.Mock()]
        e = model.Entitlement(contents=contents)
        self.assertTrue(hasattr(e, 'contents'))
        self.assertEqual(len(e.contents), 2)

        for a_content in e.contents:
            self.assertTrue(isinstance(a_content, mock.Mock))

        self.assertTrue(isinstance(e.contents[0], mock.Mock))


class EntitlementSourceBuilder(object):
    def contents_list(self, name):
        return [self.mock_content(name), self.mock_content(name)]

    def mock_content(self, name):
        """name also has to work as a label."""
        mock_content = create_mock_content(name=name)
        return mock_content

    def ent_source(self):
        cl1 = self.contents_list('content1')
        cl2 = self.contents_list('content2')

        ent1 = model.Entitlement(contents=cl1)
        ent2 = model.Entitlement(contents=cl2)

        es = model.EntitlementSource()
        es._entitlements = [ent1, ent2]
        return es


class TestContentTagMatch(fixture.SubManFixture):
    def test_empty_content_empty_product(self):
        content_tags = []
        product_tags = []
        matched = model.content_tag_match(content_tags, product_tags)
        # no requires means no missing requires
        self.assertTrue(matched)

    def test_content_empty_product(self):
        content_tags = ["awesomeos-1"]
        product_tags = []
        matched = model.content_tag_match(content_tags, product_tags)
        # content requires 'awesomeos-1" but products do not provide
        self.assertFalse(matched)

    def test_empty_content_product_awesome(self):
        content_tags = []
        product_tags = ["awesomeos-1"]
        matched = model.content_tag_match(content_tags, product_tags)
        # no content requires, so anything matches
        self.assertTrue(matched)

    def test_content_awesome_product_awesome(self):
        content_tags = ["awesomeos-1"]
        product_tags = ["awesomeos-1"]
        matched = model.content_tag_match(content_tags, product_tags)
        # require awesomeos-1, have awesomeos-1
        self.assertTrue(matched)

    def test_content_awesome_product_meh(self):
        content_tags = ["awesomeos-1"]
        product_tags = ["mehos-1"]
        matched = model.content_tag_match(content_tags, product_tags)
        # The requires awesomeos-1 is not provided
        self.assertFalse(matched)

    def test_content_multiple_product_multiple(self):
        content_tags = ["awesomeos-1", "awesomeos-server-1"]
        product_tags = ["awesomeos-1", "awesomeos-server-1"]
        matched = model.content_tag_match(content_tags, product_tags)
        # content requires awesomeos-1 and awesomeos-server-1, and they are
        # provided
        self.assertTrue(matched)

    def test_content_multiple_missing_product_multiple(self):
        content_tags = ["awesomeos-server-1", "awesomeos-1"]
        product_tags = ["awesomeos-1"]
        matched = model.content_tag_match(content_tags, product_tags)
        # content requires os and server, but only server is provided
        self.assertFalse(matched)

    def test_content_all_missing_product_multiple(self):
        content_tags = ["mehos-1", "mehos-doorstop-1"]
        product_tags = ["awesomeos-1", "awesomeos-server-1"]
        matched = model.content_tag_match(content_tags, product_tags)
        # none of the content required tags are provided
        self.assertFalse(matched)

    def test_content_dupes_product_dupes(self):
        content_tags = ["awesomeos-1", "awesomeos-1", "awesomeos-1"]
        product_tags = ["awesomeos-1", "awesomeos-1", "awesomeos-1"]
        matched = model.content_tag_match(content_tags, product_tags)
        # requires met multiple times
        self.assertTrue(matched)


class TestEntitlementSource(fixture.SubManFixture):
    def test_empty_init(self):
        es = model.EntitlementSource()
        # NOTE: this is just for testing this impl, the api
        # itself should never reference self.entitlements
        self.assertTrue(hasattr(es, '_entitlements'))

    def test(self):
        esb = EntitlementSourceBuilder()
        es = esb.ent_source()

        self.assertEqual(len(es), 2)

        for ent in es:
            self.assertTrue(isinstance(ent, model.Entitlement))
            self.assertTrue(len(ent.contents), 2)

        self.assertTrue(isinstance(es[0], model.Entitlement))


class TestFindContent(fixture.SubManFixture):
    def test(self):
        esb = EntitlementSourceBuilder()
        es = esb.ent_source()

        res = model.find_content(es, content_type="yum")
        self.assertEqual(len(res), 4)

    def test_product_tags(self):
        esb = EntitlementSourceBuilder()
        es = esb.ent_source()
        es.product_tags = []

        res = model.find_content(es, content_type="yum")
        self.assertEqual(len(res), 4)

    def test_product_tags_one_non_match(self):
        esb = EntitlementSourceBuilder()
        es = esb.ent_source()
        es.product_tags = ['something-random-tag']

        res = model.find_content(es, content_type="yum")
        self.assertEqual(len(res), 4)

    def test_product_tags_and_content_tags_match(self):
        content = create_mock_content(tags=['awesomeos-ostree-1'],
                                      content_type="ostree")
        content_list = [content]

        entitlement = model.Entitlement(contents=content_list)
        es = model.EntitlementSource()
        es._entitlements = [entitlement]
        es.product_tags = ['awesomeos-ostree-1']

        res = model.find_content(es, content_type="ostree")

        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], content)

    def test_product_tags_and_content_tags_no_match(self):
        content1 = create_mock_content(tags=['awesomeos-ostree-23'],
                                      content_type="ostree")
        content2 = create_mock_content(name="more-test-content",
                                       tags=['awesomeos-ostree-24'],
                                       content_type="ostree")
        content_list = [content1, content2]

        entitlement = model.Entitlement(contents=content_list)
        es = model.EntitlementSource()
        es._entitlements = [entitlement]
        es.product_tags = ['awesomeos-ostree-1']

        res = model.find_content(es, content_type="ostree")

        self.assertEqual(len(res), 0)

    def test_product_tags_and_content_tags_no_match_no_product_tags(self):
        content1 = create_mock_content(tags=['awesomeos-ostree-23'],
                                      content_type="ostree")
        content2 = create_mock_content(name="more-test-content",
                                       tags=['awesomeos-ostree-24'],
                                       content_type="ostree")
        content_list = [content1, content2]

        entitlement = model.Entitlement(contents=content_list)
        es = model.EntitlementSource()
        es._entitlements = [entitlement]

        res = model.find_content(es, content_type="ostree")

        self.assertEqual(len(res), 0)

    def test_product_tags_and_content_tags_no_match_empty_product_tags(self):
        content1 = create_mock_content(tags=['awesomeos-ostree-23'],
                                      content_type="ostree")
        content2 = create_mock_content(name="more-test-content",
                                       tags=['awesomeos-ostree-24'],
                                       content_type="ostree")
        content_list = [content1, content2]

        entitlement = model.Entitlement(contents=content_list)
        es = model.EntitlementSource()
        es._entitlements = [entitlement]
        es.product_tags = []

        res = model.find_content(es, content_type="ostree")

        self.assertEqual(len(res), 0)

    def test_product_tags_and_content_tags_multiple_content_one_match(self):
        content1 = create_mock_content(tags=['awesomeos-ostree-23'],
                                      content_type="ostree")
        content2 = create_mock_content(name="more-test-content",
                                       tags=['awesomeos-ostree-1'],
                                       content_type="ostree")
        content_list = [content1, content2]

        entitlement = model.Entitlement(contents=content_list)
        es = model.EntitlementSource()
        es._entitlements = [entitlement]
        es.product_tags = ['awesomeos-ostree-1']

        res = model.find_content(es, content_type="ostree")

        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], content2)

    def test_no_content_tags(self):
        content = create_mock_content(content_type="ostree")
        content_list = [content]

        entitlement = model.Entitlement(contents=content_list)
        es = model.EntitlementSource()
        es._entitlements = [entitlement]
        es.product_tags = ['awesomeos-ostree-1']

        res = model.find_content(es, content_type="ostree")
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], content)

    def test_find_container_content(self):
        container_content = create_mock_content(name="container_content",
                                                tags=["awesomeos-1"],
                                                content_type="containerImage")

        ent1 = model.Entitlement(contents=[container_content])

        ent_src = model.EntitlementSource()
        ent_src._entitlements = [ent1]
        ent_src.product_tags = ["awesomeos-1"]

        cont_list = model.find_content(ent_src,
            content_type='containerImage')
        self.assertEqual(1, len(cont_list))
        self.assertEqual('container_content', cont_list[0].name)

    def test_awesomeos_product_tags(self):
        product_tags_beta = ["awesomeos-ostree-beta", "awesomeos-ostree-beta-7"]
        product_tags_htb = ["awesomeos-ostree-htb", "awesomeos-ostree-htb-7"]
        product_tags_os = ["awesomeos-ostree", "awesomeos-ostree-7"]

        content_tags_beta = ["awesomeos-ostree-beta-7"]
        content_tags_htb = ["awesomeos-ostree-htb-7"]
        content_tags_os = ["awesomeos-ostree-7"]

        beta_ostree_content = create_mock_content(name="awesomeos_beta_ostree_content",
                                                  tags=content_tags_beta,
                                                  content_type="ostree")

        htb_ostree_content = create_mock_content(name="awesomeos_htb_ostree_content",
                                                 tags=content_tags_htb,
                                                 content_type="ostree")

        ostree_content = create_mock_content(name="awesomeos_ostree_content",
                                             tags=content_tags_os,
                                             content_type="ostree")

        ent = model.Entitlement(contents=[ostree_content,
                                          htb_ostree_content,
                                          beta_ostree_content])

        ent_src = model.EntitlementSource()
        ent_src._entitlements = [ent]

        # beta product
        ent_src.product_tags = product_tags_beta
        ostree_list = model.find_content(ent_src, content_type='ostree')
        self.assertEqual(1, len(ostree_list))
        self.assertEqual('awesomeos_beta_ostree_content', ostree_list[0].name)

        # htb products
        ent_src.product_tags = product_tags_htb
        ostree_list = model.find_content(ent_src, content_type='ostree')
        self.assertEqual(1, len(ostree_list))
        self.assertEqual('awesomeos_htb_ostree_content', ostree_list[0].name)

        ent_src.product_tags = product_tags_os
        ostree_list = model.find_content(ent_src, content_type='ostree')
        self.assertEqual(1, len(ostree_list))
        self.assertEqual('awesomeos_ostree_content', ostree_list[0].name)

        # beta product, no beta content
        ent = model.Entitlement(contents=[ostree_content])
        ent_src._entitlements = [ent]
        ent_src.product_tags = product_tags_beta
        ostree_list = model.find_content(ent_src, content_type='ostree')
        self.assertEqual(0, len(ostree_list))

        # awesomeos product, no os content, beta, htb content
        ent = model.Entitlement(contents=[htb_ostree_content,
                                          beta_ostree_content])
        ent_src._entitlements = [ent]
        ent_src.product_tags = product_tags_os
        ostree_list = model.find_content(ent_src, content_type='ostree')
        self.assertEqual(0, len(ostree_list))

        # awesomeos and beta product, all content
        ent = model.Entitlement(contents=[ostree_content,
                                          htb_ostree_content,
                                          beta_ostree_content])
        ent_src._entitlements = [ent]
        ent_src.product_tags = product_tags_os + product_tags_beta
        ostree_list = model.find_content(ent_src, content_type='ostree')
        # should be os and beta content
        self.assertEqual(2, len(ostree_list))
        content_names = [x.name for x in ostree_list]
        self.assertTrue('awesomeos_ostree_content' in content_names)
        self.assertTrue('awesomeos_beta_ostree_content' in content_names)
        self.assertFalse('awesomeos_htb_ostree_content' in content_names)

    def test_find_content_everything(self):
        yum_content = create_mock_content(name="yum_content",
                                          tags=["awesomeos-1"],
                                          content_type="yum")
        container_content = create_mock_content(name="container_content",
                                                tags=["awesomeos-1"],
                                                content_type="containerImage")
        ostree_content = create_mock_content(name="ostree_content",
                                             tags=["awesomeos-1"],
                                             content_type="ostree")

        ent1 = model.Entitlement(contents=[yum_content])
        ent2 = model.Entitlement(contents=[container_content, ostree_content])

        ent_src = model.EntitlementSource()
        ent_src._entitlements = [ent1, ent2]
        ent_src.product_tags = ["awesomeos-1"]

        yum_list = model.find_content(ent_src, content_type='yum')
        self.assertEqual(1, len(yum_list))
        self.assertEqual('yum_content', yum_list[0].name)

        cont_list = model.find_content(ent_src,
            content_type='containerImage')
        self.assertEqual(1, len(cont_list))
        self.assertEqual('container_content', cont_list[0].name)

        ostree_list = model.find_content(ent_src,
                                         content_type="ostree")
        self.assertEqual(1, len(ostree_list))
        self.assertEqual('ostree_content', ostree_list[0].name)



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2019 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import logging

from rhsmlib.facts import collector

try:
    from insights_client import constants as insights_constants
except ImportError:
    insights_constants = None

log = logging.getLogger(__name__)


class InsightsCollector(collector.FactsCollector):
    """
    Class used for collecting facts related to Red Hat Access Insights
    """

    def __init__(self, arch=None, prefix=None, testing=None, collected_hw_info=None):
        super(InsightsCollector, self).__init__(
            arch=arch,
            prefix=prefix,
            testing=testing,
            collected_hw_info=collected_hw_info
        )

        self.hardware_methods = [
            self.get_insights_machine_id
        ]

    def get_insights_machine_id(self):
        """
        Try to return content of insights machine_id (UUID)
        :return: dictionary containing insights_id, when machine_id file exist.
        Otherwise empty dictionary is returned.
        """
        insights_id = {}
        paths_to_check = [
            "/etc/insights-client/machine-id",  # should be the current known location
            "/etc/redhat-access-insights/machine-id",  # location prior to 3.0.13 of insights-client
        ]
        if insights_constants is not None and hasattr(insights_constants, 'machine_id_file'):
            paths_to_check.insert(0, insights_constants.machine_id_file)

        for filepath in paths_to_check:
            try:
                with open(filepath, "r") as fd:
                    machine_id = fd.read()
            except IOError as err:
                log.debug("Unable to read insights machine_id file: %s, error: %s" % (filepath, err))
            else:
                insights_id = {
                    "insights_id": machine_id
                }
                break

        return insights_id



from __future__ import print_function, division, absolute_import

# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
try:
    import unittest2 as unittest
except ImportError:
    import unittest

from .modelhelpers import create_pool, create_attribute_list
from subscription_manager.jsonwrapper import PoolWrapper


class TestPoolWrapper(unittest.TestCase):

    def _create_wrapper(self, add_is_virt_only=False, is_virt_only_value="true",
                        add_stacking_id=False, stacking_id=None, pool_type=None,
                        add_management_enabled=False, management_enabled_value="true"):
        attrs = {}
        if add_is_virt_only:
            attrs['virt_only'] = is_virt_only_value

        prod_attrs = {}
        if add_stacking_id:
            prod_attrs['stacking_id'] = stacking_id

        if add_management_enabled:
            prod_attrs['management_enabled'] = management_enabled_value

        calculatedAttributes = None
        if pool_type:
            calculatedAttributes = {'compliance_type': pool_type}
        pool = create_pool("pid", "pname", attributes=create_attribute_list(attrs),
                           productAttributes=create_attribute_list(prod_attrs),
                           calculatedAttributes=calculatedAttributes)
        return PoolWrapper(pool)

    def test_is_not_virt_only_when_attribute_is_false(self):
        wrapper = self._create_wrapper(add_is_virt_only=True, is_virt_only_value="false")
        self.assertFalse(wrapper.is_virt_only())

    def test_is_virt_only_when_attribute_is_true(self):
        wrapper = self._create_wrapper(add_is_virt_only=True, is_virt_only_value="true")
        self.assertTrue(wrapper.is_virt_only())

    def test_is_virt_only_when_attribute_is_1(self):
        wrapper = self._create_wrapper(add_is_virt_only=True, is_virt_only_value="1")
        self.assertTrue(wrapper.is_virt_only())

    def test_is_not_virt_only_when_attribute_is_0(self):
        wrapper = self._create_wrapper(add_is_virt_only=True, is_virt_only_value="0")
        self.assertFalse(wrapper.is_virt_only())

    def test_is_virt_only_when_attribute_is_not_set(self):
        wrapper = self._create_wrapper()
        self.assertFalse(wrapper.is_virt_only())

    def test_get_stacking_id_when_attribute_is_set(self):
        wrapper = self._create_wrapper(add_stacking_id=True, stacking_id="1234")
        self.assertEqual("1234", wrapper.get_stacking_id())

    def test_get_stacking_id_when_attribute_not_set(self):
        wrapper = self._create_wrapper(add_stacking_id=True)
        self.assertEqual(None, wrapper.get_stacking_id())

    def test_none_when_stacking_id_empty(self):
        wrapper = self._create_wrapper(add_stacking_id=True, stacking_id="")
        self.assertEqual(None, wrapper.get_stacking_id())

    def test_compliance_type(self):
        wrapper = self._create_wrapper(pool_type='double stackable')
        self.assertEqual('double stackable', wrapper.get_pool_type())

    def test_no_compliance_type(self):
        wrapper = self._create_wrapper()
        self.assertEqual('', wrapper.get_pool_type())

    def test_management_enabled_when_attribute_is_false(self):
        wrapper = self._create_wrapper(add_management_enabled=True, management_enabled_value="false")
        self.assertFalse(wrapper.management_enabled())

    def test_management_enabled_when_attribute_is_true(self):
        wrapper = self._create_wrapper(add_management_enabled=True, management_enabled_value="true")
        self.assertTrue(wrapper.management_enabled())

    def test_management_enabled_when_attribute_is_1(self):
        wrapper = self._create_wrapper(add_management_enabled=True, management_enabled_value="1")
        self.assertTrue(wrapper.management_enabled())

    def test_management_enabled_when_attribute_is_0(self):
        wrapper = self._create_wrapper(add_management_enabled=True, management_enabled_value="0")
        self.assertFalse(wrapper.management_enabled())

    def test_management_enabled_when_attribute_is_not_set(self):
        wrapper = self._create_wrapper()
        self.assertFalse(wrapper.management_enabled())



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import six

from subscription_manager import i18n_optparse


class TestWrappedIndentedHelpFormatter(unittest.TestCase):
    def setUp(self):
        self.hf = i18n_optparse.WrappedIndentedHelpFormatter(width=50)
        self.parser = i18n_optparse.OptionParser(description=u"test", formatter=self.hf)
        self.parser.add_option(u"-t", u"--test", dest=u"test",
                               default=None,
                               help=u"このシステム用に権利があるレポジトリの一覧表示このシステム用に権利があるレポジトリの一覧表示")

    def test_format_option(self):
        # use the new formatter, check for a result that we can decode to utf
        fh = self.parser.format_option_help(self.hf)
        self.assertIsInstance(fh, six.text_type)

    def test_format_usage(self):
        # optparses default format_usage uses lower cases
        # usage on 2.4, upper case on 2.6. We include our
        # own for consistency
        fu = self.hf.format_usage("%%prog [OPTIONS]")
        self.assertEqual(fu[:6], "Usage:")



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import
#
# Copyright (c) 2018 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

from .base import SyspurposeTestBase, write_to_file_utf8, Capture
import io
import json
import os
import mock

from syspurpose import files, utils
from syspurpose.files import detect_changed, three_way_merge, UNSUPPORTED, SyncedStore, SyncResult


class SyspurposeStoreTests(SyspurposeTestBase):

    def test_new_syspurpose_store(self):
        """
        A smoke test to ensure nothing bizarre happens on SyspurposeStore object creation
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        self.assertEqual(syspurpose_store.contents, {})
        self.assertEqual(syspurpose_store.path, temp_dir)

    def test_read_file_non_existent_file(self):
        """
        Can the SyspurposeStore.read_file method handle attempting to read a file which does not exist?
        """
        temp_file = os.path.join(self._mktmp(), 'syspurpose_file.json')
        self.assertFalse(os.path.exists(temp_file))

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_file)
        res = self.assertRaisesNothing(syspurpose_store.read_file)
        self.assertFalse(bool(res))

    def test_read_file_existent_file(self):
        """
        The SyspurposeStore.read_file method should return True if the file was successfully read.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        self.assertTrue(os.path.exists(temp_dir))

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        res = self.assertRaisesNothing(syspurpose_store.read_file)
        self.assertTrue(bool(res))

    def test_read_file_with_unicode_content(self):
        """
        The SyspurposeStore.read_file method should return True if the file with unicode content was successfully read.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {'key1': u'Νίκος', 'key2': [u'value_with_ř']}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        self.assertTrue(os.path.exists(temp_dir))

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        res = self.assertRaisesNothing(syspurpose_store.read_file)
        self.assertTrue(bool(res))

    def _read_file(self, file_contents=None, expected_contents=None):
        """
        Utility method for logic common to the *read_file* tests.
        :param file_contents:
        :param expected_contents:
        :return: None
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            if file_contents and not isinstance(file_contents, str):
                utils.write_to_file_utf8(f, file_contents)
            else:
                f.write(utils.make_utf8(file_contents or ''))
            f.flush()
        self.assertTrue(os.path.exists(temp_dir), "Unable to create test file in temp dir")

        # Actually do the test
        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        self.assertRaisesNothing(syspurpose_store.read_file)
        self.assertEqual(syspurpose_store.contents, expected_contents)

    def test_read_file_empty_file(self):
        """
        Can the SyspurposeStore.read_file method handle attempting to read an empty file?
        """
        self._read_file(file_contents='', expected_contents={})

    def test_read_file_non_empty(self):
        """
        Lets see if we can read a file that is non-empty
        """
        test_data = {"arbitrary": "data"}
        self._read_file(file_contents=test_data, expected_contents=test_data)

    def test_create(self):
        """
        Verify that the create method will create the directory (if needed), and that the resulting \
        file in the directory is writable by us.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        self.assertRaisesNothing(syspurpose_store.create)
        # We should have a new file in the temp_dir that we can access for writing
        self.assertTrue(os.path.exists(temp_dir))
        self.assertTrue(os.access(temp_dir, os.W_OK))

    def test_add(self):
        """
        Verify that the add method of SyspurposeStore is able to add items to lists of items
        in the store, whether they existed prior or not.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": ["preexisting_value"]}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Brand new unseen key is added
        res = self.assertRaisesNothing(syspurpose_store.add, "new_key", "new_value")
        self.assertIn("new_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["new_key"], ["new_value"])
        self.assertTrue(res, "The add method should return true when the store has changed")

        # Add to an already seen existing key
        res = self.assertRaisesNothing(syspurpose_store.add, "already_present_key", "new_value_2")
        self.assertIn("already_present_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["already_present_key"], ["preexisting_value", "new_value_2"])
        self.assertTrue(res, "The add method should return true when the store has changed")

    def test_add_new_value_to_key_with_null_value(self):
        """
        Verify that the add method of SyspurposeStore is able to add item to key with
        null value
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": None}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Brand new unseen key is added
        res = self.assertRaisesNothing(syspurpose_store.add, "already_present_key", "new_value")
        self.assertIn("already_present_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["already_present_key"], ["new_value"])
        self.assertTrue(res, "The add method should return true when the store has changed")

    def test_add_does_not_override_existing_scalar_value(self):
        """
        Verify that the add method of SyspurposeStore is able to add items to a property
        in the store, without overriding an existing scalar value the property might already contain.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": "preexisting_scalar_value"}
        with open(temp_dir, 'w') as f:
            json.dump(test_data, f)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Add to an already seen existing key, that contains a scalar value
        res = self.assertRaisesNothing(syspurpose_store.add, "already_present_key", "new_value_2")
        self.assertIn("already_present_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["already_present_key"], ["preexisting_scalar_value", "new_value_2"])
        self.assertTrue(res, "The add method should return true when the store has changed")

    def test_add_with_unicode_strings(self):
        """
        Verify that the add method of SyspurposeStore is able to add unicode strings to lists of items
        in the store.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {u'ονόματα': [u'Νίκος']}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Add to an already seen existing key
        res = self.assertRaisesNothing(syspurpose_store.add, 'ονόματα', 'Κώστας')
        self.assertIn(u'ονόματα', syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents[u'ονόματα'], [u'Νίκος', u'Κώστας'])
        self.assertTrue(res, "The add method should return true when the store has changed")

    def test_add_does_not_duplicate_existing_value(self):
        """
        Verify that the add method of SyspurposeStore will not add an item to a list, if that list already contains
        the item we're trying to add.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": ["preexisting_value"]}
        with open(temp_dir, 'w') as f:
            json.dump(test_data, f)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Try to add a value that already exists to an already seen existing key
        self.assertRaisesNothing(syspurpose_store.add, "already_present_key", "preexisting_value")
        self.assertIn("already_present_key", syspurpose_store.contents)
        self.assertEqual(len(syspurpose_store.contents["already_present_key"]), 1)
        self.assertEqual(syspurpose_store.contents["already_present_key"], ["preexisting_value"])

    def test_remove(self):
        """
        Verify that the remove method can remove items from the store.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": ["preexisting_value"]}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Remove an item from the store which we have previously seen
        res = self.assertRaisesNothing(syspurpose_store.remove, "already_present_key", "preexisting_value")
        self.assertIn("already_present_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["already_present_key"], [])
        self.assertTrue(res, "The remove method should return true when the store has changed")

        # Try to remove an item that we've previously not seen
        res = self.assertRaisesNothing(syspurpose_store.remove, "new_key", "any_value")
        self.assertFalse(res, "The remove method should return false when the store has not changed")

    def test_remove_unsets_existing_scalar_value(self):
        """
        Verify that the remove_command, in case the value specified for removal is a scalar/non-list value,
        unsets this value.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": "preexisting_scalar_value"}
        with open(temp_dir, 'w') as f:
            json.dump(test_data, f)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Remove an item from the store which we have previously seen, whose value is scalar / not contained in a list
        res = self.assertRaisesNothing(syspurpose_store.remove, "already_present_key", "preexisting_scalar_value")
        self.assertNotIn("already_present_key", syspurpose_store.contents)
        self.assertTrue(res, "The remove method should return true when the store has changed")

    def test_remove_with_unicode_strings(self):
        """
        Verify that the remove method of SyspurposeStore is able to remove unicode strings from lists of items
        in the store.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {u'ονόματα': [u'Νίκος', u'Κώστας']}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Remove from an already seen existing key
        res = self.assertRaisesNothing(syspurpose_store.remove, 'ονόματα', 'Κώστας')
        self.assertIn(u'ονόματα', syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents[u'ονόματα'], [u'Νίκος'])
        self.assertTrue(res, "The add method should return true when the store has changed")

    def test_unset(self):
        """
        Verify the operation of the unset method of SyspurposeStore
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": ["preexisting_value"]}

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        res = self.assertRaisesNothing(syspurpose_store.unset, "already_present_key")
        # We expect a value of true from this method when the store was changed
        self.assertTrue(res, "The unset method should return true when the store has changed")
        self.assertNotIn("already_present_key", syspurpose_store.contents, msg="Expected the key to no longer be present")

        res = self.assertRaisesNothing(syspurpose_store.unset, "unseen_key")
        # We expect falsey values when the store was not modified
        self.assertFalse(res, "The unset method should return false when the store has not changed")
        self.assertNotIn("unseen_key", syspurpose_store.contents, msg="The key passed to unset, has been added to the store")

    def test_unset_with_unicode_strings(self):
        """
        Verify that the unset method of SyspurposeStore is able to unset unicode strings from items
        in the store.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {u'ονόματα': [u'Νίκος']}
        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        res = self.assertRaisesNothing(syspurpose_store.unset, "ονόματα")
        # We expect a value of true from this method when the store was changed
        self.assertTrue(res, "The unset method should return true when the store has changed")
        self.assertNotIn(u'ονόματα', syspurpose_store.contents, msg="Expected the key to no longer be present")

        res = self.assertRaisesNothing(syspurpose_store.unset, 'άκυρο_κλειδί')
        # We expect falsey values when the store was not modified
        self.assertFalse(res, "The unset method should return false when the store has not changed")
        self.assertNotIn(u'άκυρο_κλειδί', syspurpose_store.contents, msg="The key passed to unset, has been added to the store")

    def test_unset_sla(self):
        """
        Verify the unset operation handles the special case for the SLA field
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"service_level_agreement": "preexisting_value"}

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        res = self.assertRaisesNothing(syspurpose_store.unset, "service_level_agreement")
        # We expect a value of true from this method when the store was changed
        self.assertTrue(res, "The unset method should return true when the store has changed")
        self.assertIn("service_level_agreement", syspurpose_store.contents, msg="Expected the key to still be in the contents, but reset to an empty string")
        # We expect the item to have been unset to None
        self.assertEqual(syspurpose_store.contents["service_level_agreement"], '')

    def test_set(self):
        """
        Verify the operation of the set method of SyspurposeStore
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"already_present_key": "old_value"}

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Check the behaviour of manipulating an existing key with an identical value (no update)
        res = self.assertRaisesNothing(syspurpose_store.set, "already_present_key", "old_value")
        self.assertFalse(res, "When a value is not actually changed, set should return false")
        self.assertIn("already_present_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["already_present_key"], "old_value")

        # Modify existing item
        res = self.assertRaisesNothing(syspurpose_store.set, "already_present_key", "new_value")
        self.assertTrue(res, "When an item is set to a new value, set should return true")
        self.assertIn("already_present_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["already_present_key"], "new_value")

        # Add new item set to a new value
        res = self.assertRaisesNothing(syspurpose_store.set, "new_key", "new_value_2")
        self.assertTrue(res, "When an item is set to a new value, set should return true")
        self.assertIn("new_key", syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents["new_key"], "new_value_2")

    def test_set_with_unicode_strings(self):
        """
        Verify the operation of the set method of SyspurposeStore when using unicode strings
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {u'ονόματα': u'Νίκος'}

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        # Check the behaviour of manipulating an existing key with an identical value (no update)
        res = self.assertRaisesNothing(syspurpose_store.set, 'ονόματα', 'Νίκος')
        self.assertFalse(res, "When a value is not actually changed, set should return false")
        self.assertIn(u'ονόματα', syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents[u'ονόματα'], u'Νίκος')

        # Modify existing item
        res = self.assertRaisesNothing(syspurpose_store.set, 'ονόματα', 'Κώστας')
        self.assertTrue(res, "When an item is set to a new value, set should return true")
        self.assertIn(u'ονόματα', syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents[u'ονόματα'], u'Κώστας')

        # Add new item set to a new value
        res = self.assertRaisesNothing(syspurpose_store.set, 'καινούργιο', 'Άλεξανδρος')
        self.assertTrue(res, "When an item is set to a new value, set should return true")
        self.assertIn(u'καινούργιο', syspurpose_store.contents)
        self.assertEqual(syspurpose_store.contents[u'καινούργιο'], u'Άλεξανδρος')

    def test_write(self):
        """
        Verify that the SyspurposeStore can write changes to the expected file.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"arbitrary_key": "arbitrary_value"}
        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        self.assertRaisesNothing(syspurpose_store.write)

        with io.open(temp_dir, 'r', encoding='utf-8') as f:
            actual_contents = self.assertRaisesNothing(json.load, f, encoding='utf-8')

        self.assertDictEqual(actual_contents, test_data)

    def test_write_with_unicode_content(self):
        """
        Verify that the SyspurposeStore can write changes that include unicode strings to the expected file.
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {u'όνομα': u'Νίκος'}
        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore, temp_dir)
        syspurpose_store.contents = dict(**test_data)

        self.assertRaisesNothing(syspurpose_store.write)

        with io.open(temp_dir, 'r', encoding='utf-8') as f:
            actual_contents = self.assertRaisesNothing(json.load, f, encoding='utf-8')

        self.assertDictEqual(actual_contents, test_data)

    def test_read(self):
        """
        Does read properly initialize a new SyspurposeStore?
        """
        temp_dir = os.path.join(self._mktmp(), 'syspurpose_file.json')
        test_data = {"arbitrary_key": "arbitrary_value"}

        with io.open(temp_dir, 'w', encoding='utf-8') as f:
            utils.write_to_file_utf8(f, test_data)

        syspurpose_store = self.assertRaisesNothing(files.SyspurposeStore.read, temp_dir)
        self.assertDictEqual(syspurpose_store.contents, test_data)


class TestSyncedStore(SyspurposeTestBase):
    """
    Tests to show the proper overall syncing functionality of the SyncedStore class.
    This class (at the time of writing) is responsible for maintaining the local syspurpose file,
    the local syspurpose cache file and retriving and syncing the syspurpose values from the server
    should the system be able to.
    """

    # These are the values we typically expect the server will return when no syspurpose is set.
    default_remote_values = {
        "role": None,
        "usage": None,
        "addOns": [],
        "serviceLevel": u""
    }

    def setUp(self):
        self.temp_dir = self._mktmp()
        self.temp_cache_dir = self._mktmp()
        # For these tests we want to make sure that the paths that are used are our mock files

        user_syspurpose_dir_patch = mock.patch('syspurpose.files.USER_SYSPURPOSE_DIR', self.temp_dir)
        user_syspurpose_dir_patch.start()
        self.addCleanup(user_syspurpose_dir_patch.stop)

        cache_dir_patch = mock.patch('syspurpose.files.CACHE_DIR', self.temp_cache_dir)
        cache_dir_patch.start()
        self.addCleanup(cache_dir_patch.stop)

        self.local_syspurpose_file = os.path.join(self.temp_dir, 'syspurpose.json')
        self.cache_syspurpose_file = os.path.join(self.temp_cache_dir, 'cache.json')

        # For these tests we want to make sure that the paths that are used are our mock files
        synced_store_local_patch = mock.patch('syspurpose.files.SyncedStore.PATH',
                                              self.local_syspurpose_file)
        synced_store_local_patch.start()
        self.addCleanup(synced_store_local_patch.stop)

        synced_store_cache_patch = mock.patch('syspurpose.files.SyncedStore.CACHE_PATH',
                                              self.cache_syspurpose_file)
        synced_store_cache_patch.start()
        self.addCleanup(synced_store_cache_patch.stop)

        self.uep = mock.Mock()
        self.uep.getConsumer.return_value = self.default_remote_values

        # Fake that the connected server supports syspurpose
        self.uep.has_capability = mock.Mock(side_effect=lambda x: x in ['syspurpose'])

        self.uep.getOwner.return_value = {
            "created": "2020-06-22T13:57:27+0000",
            "updated": "2020-06-22T13:57:27+0000",
            "id": "ff80808172dc51a10172dc51cb3e0004",
            "key": "admin",
            "displayName": "Admin Owner",
            "parentOwner": None,
            "contentPrefix": None,
            "defaultServiceLevel": None,
            "upstreamConsumer": None,
            "logLevel": None,
            "autobindDisabled": None,
            "autobindHypervisorDisabled": None,
            "contentAccessMode": "entitlement",
            "contentAccessModeList": "entitlement",
            "lastRefreshed": None,
            "href": "/owners/admin"
        }

        self.uep.getOwnerSyspurposeValidFields.return_value = {
            "owner": {
                "id": "ff80808172dc51a10172dc51cb3e0004",
                "key": "admin",
                "displayName": "Admin Owner",
                "href": "/owners/admin"
            },
            "systemPurposeAttributes": {
                "addons": [
                    "ADDON1",
                    "ADDON3",
                    "ADDON2"
                ],
                "usage": [
                    "Production",
                    "Development"
                ],
                "roles": [
                    "SP Starter",
                    "SP Server"
                ],
                "support_level": [
                    "Full-Service",
                    "Super",
                    "Layered",
                    "Standard",
                    "Premium",
                    "None"
                ]
            }
        }

    def test_falsey_values_removed_from_local_empty_local(self):
        # The falsey values ([], "", {}, None) should never end up after a SyncedStore.sync in
        # the local syspurpose file.
        self._assert_falsey_values_removed_from_local(remote_contents=self.default_remote_values,
                                                      local_contents={},
                                                      cache_contents={})

    def test_falsey_values_removed_from_local_with_local_values(self):
        # The falsey values ([], "", {}, None) should never end up after a SyncedStore.sync in
        # the local syspurpose file. Even if they were added to the local file itself
        self._assert_falsey_values_removed_from_local(remote_contents=self.default_remote_values,
                                                      local_contents={"role": ""},
                                                      cache_contents={})

    def test_falsey_values_removed_from_local_empty_strings_from_remote(self):
        # The falsey values ([], "", {}, None) should never end up after a SyncedStore.sync in
        # the local syspurpose file.

        remote_content = {
            "role": "",
            "usage": "",
            "serviceLevel": "",
            "addOns": []
        }
        self._assert_falsey_values_removed_from_local(remote_contents=remote_content,
                                                      local_contents={},
                                                      cache_contents={})

    def _assert_falsey_values_removed_from_local(self, remote_contents, local_contents, cache_contents):
        self.uep.getConsumer.return_value = remote_contents

        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), local_contents)
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), cache_contents)

        synced_store = SyncedStore(self.uep, consumer_uuid="something")
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))
        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        # All the values from the local file should be truthy.
        self.assertTrue(all(local_result[key] for key in local_result))

        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))
        # The cache should contain the entire set of values from the SyncResult
        self.assertDictEqual(cache_result, result.result)

    def test_list_items_are_order_agnostic(self):
        addons = [1, 2]
        self.uep.getConsumer.return_value = {'addOns': addons}

        # Write an out of order list to both the local and cache
        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), {'addons': addons[::-1]})
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), {'addons': addons[::-1]})

        synced_store = SyncedStore(self.uep, consumer_uuid="something")
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        self.assertSetEqual(set(result.result['addons']), set(local_result['addons']),
                            'Expected local file to have the same set of addons as the result')
        self.assertSetEqual(set(result.result['addons']),  set(cache_result['addons']),
                            'Expected cache file to have the same set of addons as the result')

    def test_server_side_falsey_removes_value_locally(self):
        initial_syspurpose = {'role': 'something'}
        remote_contents = {'role': ''}
        self.uep.getConsumer.return_value = remote_contents

        # Write an out of order list to both the local and cache
        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), initial_syspurpose)
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), initial_syspurpose)

        synced_store = SyncedStore(self.uep, consumer_uuid="something")
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        self.assertTrue('role' not in local_result,
                        'The role was falsey and should not have been in the local file')
        self.assertTrue('role' in cache_result and cache_result['role'] == remote_contents['role'],
                        'Expected the cache file to contain the same value for role as the remote')

    def test_values_not_known_server_side_are_left_alone(self):
        cache_contents = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'service_level_agreement': u'',
            u'addons': []
        }
        local_contents = {
            u'role': cache_contents[u'role'],
            u'usage': cache_contents[u'usage'],
            u'made_up_key': u'arbitrary_value'  # this key was added and is not known
        }
        remote_contents = {
            u'role': u'remote_role',
            u'usage': u'',  # Usage has been reset on the server side, should be removed locally
            u'serviceLevel': u'',
            u'addOns': []
        }

        self.uep.getConsumer.return_value = remote_contents

        # Write an out of order list to both the local and cache
        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), local_contents)
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), cache_contents)

        synced_store = SyncedStore(self.uep, consumer_uuid="something")
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        expected_local = {
            u'role': remote_contents['role'],
            u'made_up_key': local_contents['made_up_key']
        }
        expected_cache = {
            u'role': remote_contents['role'],
            u'usage': remote_contents['usage'],
            u'made_up_key': local_contents['made_up_key'],
            u'addons': [],
            u'service_level_agreement': u''
        }

        self.assert_equal_dict(expected_local, local_result)
        self.assert_equal_dict(expected_cache, cache_result)

    def test_same_values_not_synced_with_server(self):
        cache_contents = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'service_level_agreement': u'initial_sla'
        }
        local_contents = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'service_level_agreement': u'initial_sla'
        }
        remote_contents = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'serviceLevel': u'initial_sla'
        }

        self.uep.getConsumer.return_value = remote_contents
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), cache_contents)
        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), local_contents)

        synced_store = SyncedStore(self.uep, consumer_uuid="something")
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))

        # When values are still the same, then client should not try to sync
        # same values with server
        self.uep.updateConsumer.assert_not_called()

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        expected_local = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'service_level_agreement': u'initial_sla'
        }
        expected_cache = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'service_level_agreement': u'initial_sla',
            u'addons': None
        }

        self.assert_equal_dict(expected_cache, cache_result)
        self.assert_equal_dict(expected_local, local_result)

    def test_server_does_not_support_syspurpose(self):
        # This is how we detect if we have syspurpose support
        self.uep.has_capability = mock.Mock(side_effect=lambda x: x in [])

        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), {u'role': u'initial'})
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), {})

        synced_store = SyncedStore(self.uep, consumer_uuid="something")

        self.assertRaisesNothing(synced_store.set, u'role', u'new_role')
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        # The cache should not be updated at all
        self.assert_equal_dict({}, cache_result)
        self.assert_equal_dict({u'role': u'new_role'}, local_result)

        self.uep.updateConsumer.assert_not_called()

    def test_server_setting_unsupported_value(self):
        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), {u'role': u''})
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), {})

        synced_store = SyncedStore(self.uep, consumer_uuid="something", use_valid_fields=True)

        with Capture() as captured:
            synced_store.set(u'role', u'new_role')
            self.assertTrue('Warning: Provided value "new_role" is not included in the list of valid values for attribute role' in captured.out)
            self.assertTrue("SP Starter" in captured.out)
            self.assertTrue("SP Server" in captured.out)

    def test_server_setting_unsupported_key(self):
        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), {u'role': u''})
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), {})

        synced_store = SyncedStore(self.uep, consumer_uuid="something", use_valid_fields=True)

        with Capture() as captured:
            synced_store.set(u'foo', u'bar')
            self.assertTrue('Warning: Provided key "foo" is not included in the list of valid keys' in captured.out)
            self.assertTrue("addons" in captured.out)
            self.assertTrue("usage" in captured.out)
            self.assertTrue("role" in captured.out)
            self.assertTrue("service_level_agreement" in captured.out)

    def test_server_upgraded_to_support_syspurpose(self):
        # This one attempts to show that if a server does not support syspurpose and then does
        # after an upgrade perhaps, that we do the right thing (not unsetting all the user set
        # values

        self.uep.has_capability = mock.Mock(side_effect=lambda x: x in [])
        write_to_file_utf8(io.open(self.local_syspurpose_file, 'w'), {u'role': u'initial'})
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), {})

        synced_store = SyncedStore(self.uep, consumer_uuid="something")
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        self.assert_equal_dict({u'role': u'initial'}, local_result)
        self.assert_equal_dict({}, cache_result)

        # Now the "fake" upgrade

        self.uep.has_capability = mock.Mock(side_effect=lambda x: x in ["syspurpose"])
        self.uep.getConsumer.return_value = self.default_remote_values

        synced_store = SyncedStore(self.uep, consumer_uuid="something")
        result = self.assertRaisesNothing(synced_store.sync)

        self.assertTrue(isinstance(result, SyncResult))

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        expected_cache = {
            u'role': u'initial',  # This was set locally before and should still be
            u'usage': self.default_remote_values['usage'],
            u'service_level_agreement': self.default_remote_values['serviceLevel'],
            u'addons': self.default_remote_values['addOns']
        }

        self.assert_equal_dict({u'role': u'initial'}, local_result)
        self.assert_equal_dict(expected_cache, cache_result)

    def test_user_deletes_syspurpose_file(self):
        cache_contents = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'service_level_agreement': u'',
            u'addons': []
        }
        remote_contents = {
            u'role': u'initial_role',
            u'usage': u'initial_usage',
            u'serviceLevel': u'',
            u'addOns': []
        }
        consumer_uuid = "something"
        self.uep.getConsumer.return_value = remote_contents
        write_to_file_utf8(io.open(self.cache_syspurpose_file, 'w'), cache_contents)
        # We don't write anything to the self.local_syspurpose_file anywhere else, so not making
        # one is equivalent to removing an existing one.

        synced_store = SyncedStore(self.uep, consumer_uuid=consumer_uuid)
        result = self.assertRaisesNothing(synced_store.sync)

        expected_cache = {u'service_level_agreement': u'',
                          u'addons': []}
        expected_local = {}

        local_result = json.load(io.open(self.local_syspurpose_file, 'r'))
        cache_result = json.load(io.open(self.cache_syspurpose_file, 'r'))

        self.assert_equal_dict(expected_local, local_result)
        self.assert_equal_dict(expected_cache, cache_result)
        self.uep.updateConsumer.assert_called_once_with(consumer_uuid,
                                                        role=u"",
                                                        usage=u"",
                                                        service_level=u"",
                                                        addons=[])

    def test_read_file_non_existent_directory(self):
        """
        Test the SyspurposeStore.read_file can resurrect from situation, when directory /etc/rhsm/syspurpose
        does not exist
        """
        # Delete the temporary directory
        os.rmdir(self.temp_dir)

        consumer_uuid = "something"
        synced_store = SyncedStore(self.uep, consumer_uuid=consumer_uuid)
        local_content = self.assertRaisesNothing(synced_store.get_local_contents)
        self.assertEqual(local_content, {})

        # Make sure that the directory was created
        res = os.path.isdir(self.temp_dir)
        self.assertTrue(res)

    def test_read_file_non_existent_cache_directory(self):
        """
        Test the SyspurposeStore.read_file can resurrect from situation, when directory /var/lib/rhsm/cache
        does not exist
        """
        # Delete the temporary directory
        os.rmdir(self.temp_cache_dir)

        consumer_uuid = "something"
        synced_store = SyncedStore(self.uep, consumer_uuid=consumer_uuid)
        local_content = self.assertRaisesNothing(synced_store.get_local_contents)
        self.assertEqual(local_content, {})

        # Make sure that the directory was created
        res = os.path.isdir(self.temp_dir)
        self.assertTrue(res)


class TestDetectChange(SyspurposeTestBase):

    def test_added(self):
        """
        Shows that when a key is added to the other dictionary that the result is ChangeType.Added
        """
        key = "a"
        value = "value"
        base = {}
        other = {key: value}

        result = detect_changed(base=base, other=other, key=key)

        self.assertEqual(result, True)

    def test_removed(self):
        """
        Shows that when a key is removed from the other dict that the result is ChangeType.REMOVED
        """
        key = "a"
        value = "value"
        base = {key: value}
        other = {key: None}

        result = detect_changed(base=base, other=other, key=key)
        # For a source of "server" we should not consider the removal a change if the value is None
        self.assertEqual(result, True)

        result = detect_changed(base=base, other=other, key=key, source="local")
        # For a source of "local" we should consider the removal a change
        self.assertEqual(result, True)

    def test_absence_of_field_means_no_change(self):
        """
        Shows when the server does not support a particular type of field, local controls it.
        """
        key = "a"
        value = "value"
        base = {key: value}
        other = {}

        result = detect_changed(base=base, other=other, key=key)

        self.assertEqual(result, UNSUPPORTED)

    def test_changed(self):
        """
        Shows that when a key is changed from the other dict that the result is ChangeType.CHANGED
        """
        key = "a"
        value = "value"
        next_value = "next_value"
        base = {key: value}
        other = {key: next_value}

        result = detect_changed(base=base, other=other, key=key)

        self.assertEqual(result, True)

    def test_lists_out_of_order(self):
        """
        Shows that when two lists are out of order but contain the same elements it is not
        considered a change.
        """
        key = "a"
        value = ["value1", "value2"]
        next_value = value[::-1]
        base = {key: value}
        other = {key: next_value}

        result = detect_changed(base=base, other=other, key=key)

        self.assertEqual(result, False)

    def test_same(self):
        """
        Shows that when a key is the same as in the other dict that the result is ChangeType.SAME.
        """
        key = "a"
        value = "value"
        base = {key: value}
        other = {key: value}

        result = detect_changed(base=base, other=other, key=key)

        self.assertEqual(result, False)

    def test_same_empty(self):
        """
        Shows that when a key is not in either other or base the result is ChangeType.SAME
        """
        key = "a"
        non_existant_key = "b"
        value = "value"
        next_value = "next_value"
        base = {key: value}
        other = {key: next_value}

        result = detect_changed(base=base, other=other, key=non_existant_key, source='server')

        self.assertEqual(result, UNSUPPORTED)

        result = detect_changed(base=base, other=other, key=non_existant_key, source='local')

        self.assertEqual(result, False)


class TestThreeWayMerge(SyspurposeTestBase):

    def test_all_empty(self):
        base = {}
        remote = {}
        local = {}

        expected = {}
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_empty_base_no_conflict(self):
        base = {}
        remote = {
            "A": "remote",
            "B": None,
        }
        local = {
            "A": None,
            "B": "local",
        }

        expected = {"A": "remote", "B": "local"}
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_local_only(self):
        base = {}
        remote = {"B": None}
        local = {"B": "local"}

        expected = local
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_remote_only(self):
        base = {}
        remote = {"A": "remote"}
        local = {"A": None}

        expected = remote
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_key_removed_no_conflict(self):
        base = {"C": "base"}
        remote = {"C": None}
        local = {"C": None}

        expected = {"C": None}  # C should have the None value
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_key_removed_remote_not_in_base(self):
        base = {}
        remote = {}
        local = {"C": "local"}

        expected = local  # Expect Local to win
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_key_no_longer_supported_from_remote(self):
        base = {"C": "base"}
        remote = {}
        local = {"C": "base"}

        expected = local
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_key_unset_from_remote_with_false_value(self):
        base = {"C": "base"}
        remote = {"C": None}
        local = {"C": "base"}

        expected = {"C": None}
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_key_removed_from_local(self):
        base = {"C": "base"}
        remote = {"C": "base"}
        local = {"C": None}

        expected = {"C": None}
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

        # Now with the on_conflict param set to "local"

        expected = {"C": None}
        result = three_way_merge(local=local, base=base, remote=remote, on_conflict="local")
        self.assert_equal_dict(expected, result)

    def test_merge_no_potential_conflict(self):
        base = {"C": "base"}
        # A key included from candlepin with a falsey value means that the key is supported, but
        # that there is no value presently set for it.
        remote = {"A": "remote", "B": None, "C": "base"}
        local = {"B": "local", "C": "base"}

        expected = {"A": "remote", "B": "local", "C": "base"}
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_merge(self):
        """
        Shows that a change in only one place does not consitute a conflict.
        """
        base = {"C": "base"}
        remote = {"A": "remote", "C": "remote"}
        local = {"B": "local", "C": "base"}

        expected = {"A": "remote", "B": "local", "C": "remote"}
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

        # Now with local as the on_conflict winner
        expected = {"A": "remote", "B": "local", "C": "remote"}
        result = three_way_merge(local=local, base=base, remote=remote, on_conflict="local")
        self.assert_equal_dict(expected, result)

    def test_concurrent_modification(self):
        """
        This test shows that remote wins by default in the case of concurrent modification of
        a shared key. It also shows that the on_conflict kwarg can override this.
        """
        shared_key = "C"
        base = {shared_key: "base"}
        # Here the key "C" is changed from "base" to "remote" for remote and to "local" for local
        remote = {"A": "remote", "B": None, shared_key: "remote"}
        local = {"B": "local", shared_key: "local"}

        expected = {"A": "remote", "B": "local", shared_key: "remote"}

        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

        # Now with local set to win
        expected = {"A": "remote", "B": "local", shared_key: "local"}
        result = three_way_merge(local=local, base=base, remote=remote, on_conflict="local")
        self.assert_equal_dict(expected, result)

    def test_concurrent_modification_key_added(self):
        """
        This test shows that remote wins by default in the case of concurrent modification of
        a shared key even when the shared key is not in the base.
        It also shows that the on_conflict kwarg can override this.
        """
        shared_key = "C"
        base = {}
        # Here the key "C" is changed from "base" to "remote" for remote and to "local" for local
        remote = {"A": "remote", "B": None, shared_key: "remote"}
        local = {"B": "local", shared_key: "local"}

        expected = {"A": "remote", "B": "local", shared_key: "remote"}

        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

        # Now with local set to win
        expected = {"A": "remote", "B": "local", shared_key: "local"}
        result = three_way_merge(local=local, base=base, remote=remote, on_conflict="local")
        self.assert_equal_dict(expected, result)

    def test_merge_conflicting_lists(self):
        """
        This test shows that lists are treated atomically (as in we do not merge differing lists).
        """
        shared_key = "C"
        base = {shared_key: ["base"]}
        # Here the key "C" is changed from "base" to "remote" for remote and to "local" for local
        remote = {"A": "remote", "B": None, shared_key: ["remote"]}
        local = {"B": "local", shared_key: ["local"]}

        expected = {"A": "remote", "B": "local", shared_key: ["remote"]}
        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

        # Now with local set to win
        expected = {"A": "remote", "B": "local", shared_key: ["local"]}
        result = three_way_merge(local=local, base=base, remote=remote, on_conflict="local")
        self.assert_equal_dict(expected, result)

    def test_invalid_on_conflict_value(self):
        self.assertRaises(ValueError, three_way_merge, local={}, base={}, remote={},
                          on_conflict="oops")

    def test_merge_remote_missing_field(self):
        """
        Shows that if the server does not support a field, local gets to modify it.
        """
        base = {"B": None}
        remote = {}
        local = {"B": "local"}

        expected = {"B": "local"}

        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_merge_remote_empty_field(self):
        """
        Shows that if the server has field with empty string, local gets to modify it.
        """
        base = {"B": None}
        remote = {"B": ""}
        local = {"B": "local"}

        expected = {"B": "local"}

        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)

    def test_merge_missing_remote_list(self):
        """
        Shows that if the server does not have list, local can add it.
        """
        base = {"B": None}
        remote = {}
        local = {"B": ["local"]}

        expected = {"B": ["local"]}

        result = three_way_merge(local=local, base=base, remote=remote)
        self.assert_equal_dict(expected, result)



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
from rhsmlib.facts import collector
from rhsmlib.facts import custom
from rhsmlib.facts import host_collector
from rhsmlib.facts import hwprobe
from rhsmlib.facts import insights
from rhsmlib.facts import kpatch
from rhsmlib.facts import cloud_facts


class AllFactsCollector(collector.FactsCollector):
    def __init__(self):
        self.collectors = [
            collector.StaticFactsCollector,
            host_collector.HostCollector,
            hwprobe.HardwareCollector,
            custom.CustomFactsCollector,
            insights.InsightsCollector,
            kpatch.KPatchCollector,
            cloud_facts.CloudFactsCollector
        ]

    def get_all(self):
        results = {}
        for fact_collector_cls in self.collectors:
            fact_collector = fact_collector_cls(collected_hw_info=results)
            results.update(fact_collector.get_all())
        return results



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
try:
    import unittest2 as unittest
except ImportError:
    import unittest

from datetime import datetime, timedelta

# TODO: move to python-rhsm test suite?

from .stubs import StubProduct, StubEntitlementCertificate


def yesterday():
    now = datetime.now()
    then = now - timedelta(days=1)
    return then


class ProductTests(unittest.TestCase):

    def test_no_provided_tags(self):
        p = StubProduct('product')
        self.assertEqual(0, len(p.provided_tags))
        p = StubProduct('product', provided_tags=None)
        self.assertEqual(0, len(p.provided_tags))
        p = StubProduct('product', provided_tags="")
        self.assertEqual(0, len(p.provided_tags))

    def test_one_provided_tags(self):
        p = StubProduct('product', provided_tags="TAG1")
        self.assertEqual(1, len(p.provided_tags))
        self.assertEqual("TAG1", p.provided_tags[0])

    def test_multiple_provided_tags(self):
        p = StubProduct('product', provided_tags="TAG1,TAG2,TAG3")
        self.assertEqual(3, len(p.provided_tags))
        self.assertEqual("TAG1", p.provided_tags[0])
        self.assertEqual("TAG2", p.provided_tags[1])
        self.assertEqual("TAG3", p.provided_tags[2])


class EntitlementCertificateTests(unittest.TestCase):

    def test_valid_order_date_gives_valid_cert(self):
        cert = StubEntitlementCertificate(StubProduct('product'),
                start_date=datetime(2010, 7, 27),
                end_date=datetime(2050, 7, 26))

        self.assertTrue(cert.is_valid())

    def test_expired_order_date_gives_invalid_cert(self):
        cert = StubEntitlementCertificate(StubProduct('product'),
                start_date=datetime(2010, 7, 27),
                end_date=yesterday())

        self.assertFalse(cert.is_valid())



from __future__ import print_function, division, absolute_import

# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging

from rhsmlib.dbus import base_object
from subscription_manager.injectioninit import init_dep_injection

log = logging.getLogger(__name__)
init_dep_injection()


class Main(base_object.BaseObject):
    """Just a place holder for now"""
    pass



from __future__ import print_function, division, absolute_import

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from rhsm.connection import RestlibException
from subscription_manager.ga import Gtk as ga_Gtk
from subscription_manager.gui import utils
from subscription_manager.gui import storage
from nose.plugins.attrib import attr

# we need gtk 2.18+ to do the right markup in likify
MIN_GTK_MAJOR = 2
MIN_GTK_MINOR = 18
MIN_GTK_MICRO = 0


@attr('gui')
class TestLinkify(unittest.TestCase):
    no_url = "this does not have a url"
    https_url = "https://www.redhat.com"
    http_url = "http://www.redhat.com"
    expected_http_url = """<a href="%s">%s</a>""" % (http_url, http_url)
    expected_https_url = """<a href="%s">%s</a>""" % (https_url, https_url)

    http_url_dash = "http://example.com/something-foo/blah_something/"
    expected_http_url_dash = """<a href="%s">%s</a>""" % (http_url_dash, http_url_dash)

    nested_space = """<small>http://www.redhat.com </small>"""
    nested = """<small>http://www.redhat.com</small>"""
    expected_nested = """<small><a href="%s">%s</a></small>""" % (http_url, http_url)
    expected_nested_space = """<small><a href="%s">%s</a> </small>""" % (http_url, http_url)

    example_1 = """https://access.redhat.com/kb/docs/DOC-45563"""
    example_2 = """https://www.redhat.com/wapps/sso/rhn/lostPassword.html"""
    expected_example_1 = """<a href="%s">%s</a>""" % (example_1, example_1)
    expected_example_2 = """<a href="%s">%s</a>""" % (example_2, example_2)

    if ga_Gtk.check_version(MIN_GTK_MAJOR, MIN_GTK_MINOR, MIN_GTK_MICRO):
        __test__ = False

    def test_no_url(self):
        ret = utils.linkify(self.no_url)
        self.assertEqual(ret, self.no_url)

    def test_https_url(self):
        ret = utils.linkify(self.https_url)
        self.assertEqual(ret, self.expected_https_url)

    def test_http_url(self):
        ret = utils.linkify(self.http_url)
        self.assertEqual(ret, self.expected_http_url)

    def test_http_nested_space(self):
        ret = utils.linkify(self.nested_space)
        self.assertEqual(ret, self.expected_nested_space)

    def test_nested(self):
        ret = utils.linkify(self.nested)
        self.assertEqual(ret, self.expected_nested)

    def test_dash(self):
        ret = utils.linkify(self.http_url_dash)
        self.assertEqual(ret, self.expected_http_url_dash)

    def test_example_1(self):
        ret = utils.linkify(self.example_1)
        self.assertEqual(ret, self.expected_example_1)

    def test_example_2(self):
        ret = utils.linkify(self.example_2)
        self.assertEqual(ret, self.expected_example_2)


@attr('gui')
class TestGatherGroup(unittest.TestCase):
    def test_gather_group(self):
        """
        The tree for this test looks like:
            root
                child-1
                    grandchild-1
                child-2
            root-sibling
        utils.gather_group() should return root and all its children and
        not return root-sibling.
        """
        store = storage.MappedTreeStore({'name': str})
        root_iter = store.add_map(None, {'name': 'root'})
        store.add_map(None, {'name': 'root-sibling'})
        child_1_iter = store.add_map(root_iter, {'name': 'child-1'})
        store.add_map(root_iter, {'name': 'child-2'})
        store.add_map(child_1_iter, {'name': 'grandchild-1'})

        refs = utils.gather_group(store, root_iter, [])
        names = set()
        for r in refs:
            m = r.get_model()
            names.add(m.get_value(m.get_iter(r.get_path()), 0))
        self.assertEqual(names, set(['root', 'child-1', 'child-2', 'grandchild-1']))


@attr('gui')
class TestGuiExceptionMapper(unittest.TestCase):
    def test_restlib_exception_with_markup(self):
        exception_mapper = utils.GuiExceptionMapper()
        unescaped_input = '<a=3>&'
        bad_restlib_exception = RestlibException(404, unescaped_input)
        expected_output = '&lt;a=3&gt;&amp;'
        actual_output = exception_mapper.get_message(bad_restlib_exception)
        self.assertEqual(expected_output, actual_output)



from __future__ import print_function, division, absolute_import

# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

import heapq
import itertools


class HuffmanNode(object):
    """
    Represents a node in a Huffman tree.
    """

    def __init__(self, weight, value=None, left=None, right=None, parent=None):
        """
        :param weight:  number representing the weight/priority of this node
        :type  weight:  int
        :param value:   any value carried by this node, such as a symbol to be
                        used in reconstructing (uncompressing) some data.
        :param left:    child node on the left, should have weight <= right
        :type  left:    rhsm.huffman.HuffmanNode
        :param right:   child node on the right, should have weight >= left
        :type  right:   rhsm.huffman.HuffmanNode
        :param parent:  parent node
        :type  parent:  rhsm.huffman.HuffmanNode
        """
        self.weight = weight
        self.value = value
        self.left = left
        self.right = right
        self.parent = parent

    @classmethod
    def combine(cls, left, right):
        """
        Combine two nodes according to Huffman's tree-building algorithm. The
        weight of the left node should be <= that of the right node. If weights
        are equal, left should be the node that was in the queue longer. This
        creates a new node and sets it as the parent attribute of each child.

        :param left:    child node on the left, should have weight <= right
        :type  left:    rhsm.huffman.HuffmanNode
        :param right:   child node on the right, should have weight >= left
        :type  right:   rhsm.huffman.HuffmanNode

        :return:        new node that is the combination of left and right
        :rtype:         rhsm.huffman.HuffmanNode
        """
        node = cls(left.weight + right.weight, None, left, right)
        left.parent = node
        right.parent = node
        return node

    @property
    def is_leaf(self):
        """
        :return:    True iff left and right are None, else False
        :rtype:     bool
        """
        return self.right is None and self.left is None

    @property
    def direction_from_parent(self):
        """
        :return:    '0' if self is left of its parent, or '1' if right of parent.
        :rtype:     str
        """
        if self.parent is None:
            raise AttributeError
        if self.parent.left is self:
            return '0'
        else:
            return '1'

    @property
    def code(self):
        """
        :return:    Huffman code for this node as a series of characters '0' and '1'
        :rtype:     str
        """
        if not self.is_leaf:
            raise AttributeError('node is not a leaf')
        turns = []
        next_node = self
        while next_node is not None:
            if next_node.parent is not None:
                turns.insert(0, next_node.direction_from_parent)
            next_node = next_node.parent
        return ''.join(turns)

    @classmethod
    def build_tree(cls, nodes):
        """
        :param nodes:   list of HuffmanNode instances that will become leaves
                        in a Huffman tree.
        :type  nodes:   list
        :return:        HuffmanNode instance that is the root node of the tree
        :rtype:         rhsm.huffman.HuffmanNode
        """
        # the counter makes sure that when nodes of equal weight are compared,
        # the one most recently added gets chosen
        counter = itertools.count()
        # We use the heapq module to make a min priority queue
        queue = [(node, next(counter)) for node in nodes]
        heapq.heapify(queue)
        while True:
            left, count = heapq.heappop(queue)
            try:
                right, count = heapq.heappop(queue)
            except IndexError:
                # no more nodes to compare, so a is the root node of the tree
                return left
            heapq.heappush(queue, (cls.combine(left, right), next(counter)))

    def __lt__(self, other):
        return self.weight < other.weight

    def __le__(self, other):
        return self.weight <= other.weight

    def __gt__(self, other):
        return self.weight > other.weight

    def __ge__(self, other):
        return self.weight >= other.weight

    def __eq__(self, other):
        if not hasattr(other, 'weight'):
            return False
        return self.weight == other.weight

    def __ne__(self, other):
        if not hasattr(other, 'weight'):
            return True
        return self.weight != other.weight

    def __hash__(self):
        return self.value

    def __repr__(self):
        return 'HuffmanNode(%d, "%s")' % (self.weight, self.value)



from __future__ import print_function, division, absolute_import

from mock import Mock, patch

from test.fixture import SubManFixture, set_up_mock_sp_store

from test.stubs import StubBackend, StubFacts
from subscription_manager.gui.registergui import RegisterWidget, RegisterInfo,  \
    CredentialsScreen, ActivationKeyScreen, ChooseServerScreen, AsyncBackend, \
    CREDENTIALS_PAGE, CHOOSE_SERVER_PAGE
from subscription_manager.gui.autobind import AllProductsCoveredException

from subscription_manager.ga import GObject as ga_GObject
from subscription_manager.ga import Gtk as ga_Gtk

import sys
import six
from nose.plugins.attrib import attr


@attr('gui')
class RegisterWidgetTests(SubManFixture):
    def setUp(self):
        super(RegisterWidgetTests, self).setUp()
        self.exc_infos = []
        self.excs = []
        self.backend = StubBackend()
        expected_facts = {'fact1': 'one',
                          'fact2': 'two',
                          'system': '',
                          'system.uuid': 'MOCKUUID'}
        self.facts = StubFacts(fact_dict=expected_facts)

        self.reg_info = RegisterInfo()
        self.rs = RegisterWidget(backend=self.backend, reg_info=self.reg_info)

        self.rs._screens[CHOOSE_SERVER_PAGE] = Mock()
        self.rs._screens[CHOOSE_SERVER_PAGE].index = 0
        self.rs._screens[CHOOSE_SERVER_PAGE].screens_index = 0
        self.rs._screens[CHOOSE_SERVER_PAGE].button_label = "Dummy"
        self.rs._screens[CHOOSE_SERVER_PAGE].apply.return_value = CREDENTIALS_PAGE

    def test_show(self):
        self.rs.initialize()

    def page_notify_handler(self, obj, param):
        page_after = obj.get_current_page()
        # NOTE: these exceptions are not in the nost test context,
        #       so they don't actually fail nose
        try:
            self.assertEqual(page_after, 0)
        except Exception:
            self.exc_infos.append(sys.exc_info())
            return

        self.correct_page = True
        self.gtk_quit()
        return False

    def error_handler(self, obj, msg, exc_info):
        page_after = self.rs.register_notebook.get_current_page()

        # NOTE: these exceptions are not in the nost test context,
        #       so they don't actually fail nose
        try:
            self.assertEqual(page_after, 0)
        except Exception:
            self.exc_infos.append(sys.exc_info())
            return

        self.correct_page = True
        self.gtk_quit()
        return False

    def emit_proceed(self):
        self.rs.emit('proceed')
        return False

    def emit_error(self):
        self.rs.emit('register-error', 'Some register error', None)
        return False

    # FIXME: unit tests for gtk is a weird universe
    def test_registration_error_returns_to_page(self):
        self.rs.initialize()

        self.correct_page = None

        self.rs.register_notebook.connect('notify::page', self.page_notify_handler)

        self.rs.connect('register-error', self.error_handler)

        ga_GObject.timeout_add(3000, self.gtk_quit_on_fail)
        ga_GObject.idle_add(self.emit_proceed)
        ga_GObject.idle_add(self.emit_error)

        # run till quit or timeout
        # if we get to the state we want we can call quit
        ga_Gtk.main()

        # If we saw any exceptions, raise them now so we fail nosetests
        for exc_info in self.exc_infos:
            six.reraise(*exc_info)

        self.assertTrue(self.correct_page)

    # if we got the right answer, go ahead and end gtk.main()
    def gtk_quit(self):
        ga_Gtk.main_quit()

    # End the main loop, but first add an exception to sys.exc_info so
    # the end of the tests can fail on it.
    def gtk_quit_on_fail(self):
        try:
            self.fail("registergui didn't get a signal before the timeout.")
        except Exception:
            self.exc_infos.append(sys.exc_info())

        ga_Gtk.main_quit()

    def test_screen_history_empty(self):
        self.rs.initialize()
        self.assertTrue(self.rs.applied_screen_history.is_empty())
        self.rs.apply_current_screen()
        self.assertFalse(self.rs.applied_screen_history.is_empty())
        self.rs._pop_last_screen()
        self.assertTrue(self.rs.applied_screen_history.is_empty())


def mock_parent():
    parent = Mock()
    backend = StubBackend()
    parent.backend = backend
    parent.async_backend = Mock()


class StubReg(object):
    def __init__(self):
        self.parent_window = Mock()
        self.backend = StubBackend()
        self.async_backend = Mock()
        self.reg_info = Mock()
        self.expected_facts = {'fact1': 'one',
                               'fact2': 'two',
                               'system': '',
                               'system.uuid': 'MOCKUUID'}
        self.facts = StubFacts(fact_dict=self.expected_facts)


@attr('gui')
class CredentialsScreenTests(SubManFixture):

    def setUp(self):
        super(CredentialsScreenTests, self).setUp()

        stub_reg = StubReg()
        self.screen = CredentialsScreen(reg_info=stub_reg.reg_info,
                                        async_backend=stub_reg.async_backend,
                                        parent_window=stub_reg.parent_window)

    def test_clear_credentials_dialog(self):
        # Pull initial value here since it will be different per machine.
        default_consumer_name_value = self.screen.consumer_name.get_text()
        self.screen.account_login.set_text("foo")
        self.screen.account_password.set_text("bar")
        self.screen.skip_auto_bind.set_active(True)
        self.screen.consumer_name.set_text("CONSUMER")
        self.screen.clear()
        self.assertEqual("", self.screen.account_login.get_text())
        self.assertEqual("", self.screen.account_password.get_text())
        self.assertFalse(self.screen.skip_auto_bind.get_active())
        self.assertEqual(default_consumer_name_value,
                          self.screen.consumer_name.get_text())


@attr('gui')
class ActivationKeyScreenTests(SubManFixture):
    def setUp(self):
        super(ActivationKeyScreenTests, self).setUp()
        stub_reg = StubReg()
        self.screen = ActivationKeyScreen(reg_info=stub_reg.reg_info,
                                          async_backend=stub_reg.async_backend,
                                          parent_window=stub_reg.parent_window)

    def test_split_activation_keys(self):
        expected = ['hello', 'world', 'how', 'are', 'you']
        key_input = "hello, world,how  are , you"
        result = self.screen._split_activation_keys(key_input)
        self.assertEqual(expected, result)


@attr('gui')
class ChooseServerScreenTests(SubManFixture):
    def setUp(self):
        super(ChooseServerScreenTests, self).setUp()
        stub_reg = StubReg()
        self.screen = ChooseServerScreen(reg_info=stub_reg.reg_info,
                                         async_backend=stub_reg.async_backend,
                                         parent_window=stub_reg.parent_window)

    def test_activation_key_checkbox_sensitive(self):
        self.screen.server_entry.set_text("foo.bar:443/baz")
        self.assertTrue(self.screen.activation_key_checkbox.get_property('sensitive'))

    def test_activation_key_checkbox_prod_sensitive(self):
        self.screen.server_entry.set_text("subscription.rhsm.redhat.com:443/baz")
        self.assertTrue(self.screen.activation_key_checkbox.get_property('sensitive'))

    def test_activation_key_checkbox_inactive_when_insensitive(self):
        self.screen.server_entry.set_text("foo.bar:443/baz")
        self.screen.activation_key_checkbox.set_active(True)
        self.screen.server_entry.set_text("subscription.rhsm.redhat.com:443/baz")
        self.assertTrue(self.screen.activation_key_checkbox.get_property('sensitive'))
        self.assertTrue(self.screen.activation_key_checkbox.get_property('active'))

    @patch('subscription_manager.gui.registergui.config')
    def test__on_default_button_clicked(self, config):
        config.DEFAULT_HOSTNAME = "subscription.rhsm.redhat.com"
        config.DEFAULT_PORT = '443'
        config.DEFAULT_PREFIX = "/subscription"

        non_default = "foo.bar:8443/baz"
        expected = "%s:%s%s" % (config.DEFAULT_HOSTNAME,
            config.DEFAULT_PORT,
            config.DEFAULT_PREFIX)
        self.screen.server_entry.set_text(non_default)
        self.screen._on_default_button_clicked(None)  # The widget param is not used
        result = self.screen.server_entry.get_text()
        self.assertEqual(expected, result)


@attr('gui')
class AsyncBackendTests(SubManFixture):
    def setUp(self):
        super(AsyncBackendTests, self).setUp()
        self.backend = StubBackend()
        self.asyncBackend = AsyncBackend(self.backend)
        syspurpose_patch = patch('subscription_manager.syspurposelib.SyncedStore')
        self.mock_sp_store = syspurpose_patch.start()
        self.mock_sp_store, self.mock_sp_store_contents = set_up_mock_sp_store(self.mock_sp_store)
        self.addCleanup(syspurpose_patch.stop)

    def test_auto_system_complete(self):
        self.backend.cp_provider.get_consumer_auth_cp().getConsumer = \
            Mock(return_value={"serviceLevel": "", "owner": {"key": "admin"}})
        self.backend.cs.valid_products = ['RH001', 'RH002']
        self.backend.cs.installed_products = ['RH001', 'RH002']
        self.backend.cs.partial_stacks = []
        self.backend.cs.system_status = 'valid'
        self.backend.cp_provider.get_consumer_auth_cp().getServiceLevelList = Mock(return_value=[])
        self.assertRaises(AllProductsCoveredException, self.asyncBackend._find_suitable_subscriptions, '12345')

    def test_auto_system_partial(self):
        self.backend.cp_provider.get_consumer_auth_cp().getConsumer = \
            Mock(return_value={"serviceLevel": "", "owner": {"key": "admin"}})
        self.backend.cp_provider.get_consumer_auth_cp().dryRunBind = Mock(return_value=[])
        self.backend.cs.valid_products = ['RH001', 'RH002']
        self.backend.cs.installed_products = ['RH001', 'RH002']
        self.backend.cs.partial_stacks = []
        self.backend.cs.system_status = 'partial'
        self.backend.cp_provider.get_consumer_auth_cp().getServiceLevelList = Mock(return_value=[])
        self.asyncBackend._find_suitable_subscriptions('12345')



from __future__ import print_function, division, absolute_import

#
# GUI Module for the Autobind Wizard
#
# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging

log = logging.getLogger(__name__)


class DryRunResult(object):
    """ Encapsulates a dry-run autobind result from the server. """

    def __init__(self, service_level, server_json, cert_sorter):
        self.json = server_json
        self.sorter = cert_sorter
        self.service_level = service_level

    def covers_required_products(self):
        """
        Return True if this dry-run result would cover all installed
        products which are not covered by a valid entitlement.

        NOTE: we do not require full stacking compliance here. The server
        will return the best match it can find, but that may still leave you
        only partially entitled. We will still consider this situation a valid
        SLA to use, the key point being you have access to the content you
        need.
        """
        required_products = set(self.sorter.unentitled_products.keys())

        # The products that would be covered if we did this autobind:
        autobind_products = set()

        for pool_quantity in self.json:
            pool = pool_quantity['pool']
            # This is usually the MKT product and has no content, but it
            # doesn't hurt to include it:
            autobind_products.add(pool['productId'])
            for provided_prod in pool['providedProducts']:
                autobind_products.add(provided_prod['productId'])
        log.debug("Autobind would give access to: %s" % autobind_products)
        if required_products.issubset(autobind_products):
            log.debug("Found valid service level: %s" % self.service_level)
            return True
        else:
            log.debug("Service level does not cover required products: %s" %
                      self.service_level)
            return False


class ServiceLevelNotSupportedException(Exception):
    """
    Exception for AutobindController.load. The remote candlepin doesn't
    support service levels.
    """
    pass


class AllProductsCoveredException(Exception):
    """
    Exception for AutobindController.load. The system doesn't have any
    products that are in need of entitlements.
    """
    pass


class NoProductsException(Exception):
    """
    Exception for AutobindController.load. The system has no products, and
    thus needs no entitlements.
    """
    pass



from __future__ import print_function, division, absolute_import

from gtk.gdk import color_parse
from gtk.gdk import Cursor, Event
from gtk.gdk import BUTTON_PRESS, WATCH, WINDOW_TYPE_HINT_DIALOG


class WindowTypeHint(object):
    DIALOG = WINDOW_TYPE_HINT_DIALOG


class CursorType(object):
    WATCH = WATCH


class EventType(object):
    BUTTON_PRESS = BUTTON_PRESS


enums = [CursorType, EventType, WindowTypeHint]
classes = [Cursor, Event]
methods = [color_parse]
__all__ = classes + methods + enums



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import mock

from . import fixture

from subscription_manager import healinglib


class TestHealingActionInvoker(fixture.SubManFixture):
    def setUp(self):
        super(TestHealingActionInvoker, self).setUp()

    def test_autoheal_off(self):
        mock_uep = mock.Mock()
        mock_uep.getConsumer = mock.Mock(return_value=self._consumer())
        self.set_consumer_auth_cp(mock_uep)

        hl = healinglib.HealingActionInvoker()
        hl.update()

    def test_autoheal_on(self):
        mock_uep = mock.Mock()
        consumer = {'autoheal': True}
        mock_uep.getConsumer = mock.Mock(return_value=consumer)
        self.set_consumer_auth_cp(mock_uep)

        hl = healinglib.HealingActionInvoker()
        report = hl.update()
        report.print_exceptions()

    def _consumer(self):
        consumer = {}
        return consumer


# FIXME: assert something
class TestHealingUpdateAction(fixture.SubManFixture):
    # HealingLib is very thin wrapper to HealingUpdateAction atm,
    # so basically the same tests
    def setUp(self):
        super(TestHealingUpdateAction, self).setUp()

    def test_autoheal_off(self):
        mock_uep = mock.Mock()
        # nothing set on consumer
        consumer = {}
        mock_uep.getConsumer = mock.Mock(return_value=consumer)
        self.set_consumer_auth_cp(mock_uep)

        hl = healinglib.HealingUpdateAction()
        hl.perform()

    def test_autoheal_on(self):
        mock_uep = mock.Mock()
        consumer = {'autoheal': True}
        mock_uep.getConsumer = mock.Mock(return_value=consumer)
        self.set_consumer_auth_cp(mock_uep)

        hl = healinglib.HealingUpdateAction()
        hl.perform()



from __future__ import print_function, division, absolute_import

# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
import os
import shutil

from distutils import log, dir_util
from distutils.command.build_py import build_py as _build_py
from distutils.spawn import spawn

from build_ext.utils import Utils, BaseCommand


# Courtesy http://wiki.maemo.org/Internationalize_a_Python_application
class BuildTrans(BaseCommand):
    description = 'Compile .po files into .mo files'
    user_options = _build_py.user_options + [
        ('lint', 'l', 'check po files only'),
    ]

    boolean_options = ['lint']
    app_name = "rhsm"

    def initialize_options(self):
        self.build_base = None
        self.lint = None

    def finalize_options(self):
        self.set_undefined_options('build', ('build_base', 'build_base'))

    def compile(self, src, dest):
        log.debug("Compiling %s" % src)
        if self.lint:
            dest = '/dev/null'

        cmd = ['msgfmt', '--check', '--statistics', '-o', dest, src]
        spawn(cmd)

    def merge_desktop(self, src, dest):
        log.debug("Merging desktop file %s" % src)
        if self.lint:
            dest = '/dev/null'

        cmd = ['intltool-merge', '-d', 'po', src, dest]
        spawn(cmd)

    def run(self):
        for po_file in Utils.find_files_of_type('po', '*.po'):
            lang = os.path.basename(po_file)[:-3]
            dest_path = os.path.join(self.build_base, 'locale', lang, 'LC_MESSAGES')
            dest = os.path.join(dest_path, self.app_name + '.mo')
            Utils.run_if_new(po_file, dest, self.compile)

        for desktop_file in Utils.find_files_of_type('etc-conf', '*.desktop.in'):
            output_file = os.path.basename("%s" % os.path.splitext(desktop_file)[0])

            dest_path = os.path.join(self.build_base, 'applications')
            if output_file == 'rhsm-icon.desktop':
                dest_path = os.path.join(self.build_base, 'autostart')
            dest = os.path.join(dest_path, output_file)
            Utils.run_if_new(desktop_file, dest, self.merge_desktop)


class UpdateTrans(BaseCommand):
    description = 'Update .po files with msgmerge'
    user_options = _build_py.user_options + [
        ('key-file=', 'k', 'file to update'),
    ]

    def initialize_options(self):
        self.key_file = os.path.join(os.curdir, 'po', 'keys.pot')

    def merge(self, po_file, key_file):
        log.debug("Updating %s" % po_file)
        cmd = ['msgmerge', '-N', '--backup=none', '-U', po_file, self.key_file]
        spawn(cmd)

    def run(self):
        for po_file in Utils.find_files_of_type('po', '*.po'):
            self.merge(po_file, self.key_file)


class UniqTrans(BaseCommand):
    description = 'Unify duplicate translations with msguniq'

    def run(self):
        for po_file in Utils.find_files_of_type('po', '*.po'):
            cmd = ['msguniq', po_file, '-o', po_file]
            spawn(cmd)


class Gettext(BaseCommand):
    description = 'Extract strings to po files.'
    user_options = _build_py.user_options + [
        ('lint', 'l', 'check po files only'),
        ('key-file=', 'k', 'file to write to'),
    ]

    boolean_options = ['lint']

    def initialize_options(self):
        self.build_base = None
        self.lint = None
        self.key_file = os.path.join(os.curdir, 'po', 'keys.pot')

    def finalize_options(self):
        self.src_dirs = self.distribution.package_dir
        print()
        if self.lint:
            self.key_file = '/dev/null'

    def find_py(self):
        files = []

        for src in self.src_dirs.values():
            print(src)
            files.extend(list(Utils.find_files_of_type(src, '*.py')))

        files.extend(list(Utils.find_files_of_type('bin', '*')))
        return files

    def find_c(self):
        files = []

        for src in self.src_dirs.values():
            files.extend(list(Utils.find_files_of_type(src, '*.c', '*.h')))

        files.extend(list(Utils.find_files_of_type('tmp', '*.h')))
        return files

    def find_glade(self):
        files = []
        for src in self.src_dirs.values():
            files.extend(list(Utils.find_files_of_type(src, '*.ui', '*.glade')))
        return files

    def find_js(self):
        files = []
        files.extend(list(Utils.find_files_of_type('cockpit/src', '*.js', '*.jsx')))
        return files

    def find_desktop(self):
        files = []
        files.extend(list(Utils.find_files_of_type('etc-conf', '*.desktop.in')))
        return files

    def _write_sources(self, manifest_file, find_function):
        with open(manifest_file, 'w') as f:
            f.write('\n'.join(find_function()))
            f.write('\n')

    def run(self):
        manifest_prefix = os.path.join(os.curdir, 'po', 'POTFILES')

        # Begin with a fresh key file
        dir_util.mkpath('tmp')
        if self.lint:
            tmp_key_file = self.key_file
        else:
            tmp_key_file = os.path.join('tmp', os.path.basename(self.key_file))

        # Create xgettext friendly header files from the desktop files.
        # See http://stackoverflow.com/a/23643848/6124862
        cmd = ['intltool-extract', '-l', '--type=gettext/ini']
        for desktop_file in Utils.find_files_of_type('etc-conf', '*.desktop.in'):
            spawn(cmd + [desktop_file])

        cmd = ['xgettext', '--from-code=utf-8', '--add-comments=TRANSLATORS:', '--sort-by-file', '-o', tmp_key_file]

        # These tuples contain a template for the file name that will contain a list of
        # all source files of a given type to translate, a function that finds all the
        # source files of a given type, the source language to send to xgettext, and a
        # list of any additional arguments.
        #
        # ProTip: Use extensions that won't be confused with source files.
        trans_types = [
            # The C source files require that we inform xgettext to extract strings from the
            # _() and N_() functions.
            ('%s.c_files', self.find_c, 'C', ['-k_', '-kN_']),
            ('%s.py_files', self.find_py, 'Python', []),
            ('%s.glade_files', self.find_glade, 'Glade', []),
            ('%s.js_files', self.find_js, 'JavaScript', []),
        ]

        for manifest_template, search_func, language, other_options in trans_types:
            manifest = manifest_template % manifest_prefix
            self._write_sources(manifest, search_func)

            specific_opts = ['-f', manifest, '--language', language]
            specific_opts.extend(other_options)
            if os.path.exists(tmp_key_file):
                specific_opts.append('--join-existing')
            log.debug("Running %s" % ' '.join(cmd + specific_opts))
            spawn(cmd + specific_opts)

        if not self.lint:
            shutil.copy2(tmp_key_file, self.key_file)

        # Delete the directory holding the temporary files created by intltool-extract
        # and the temporary keys.pot
        shutil.rmtree('tmp')


class GettextWithOptParse(Gettext):
    def find_py(self):
        # Can't use super since we're descended from a old-style class
        files = Gettext.find_py(self)

        # We need to grab some strings out of optparse for translation
        import optparse
        optparse_source = "%s.py" % os.path.splitext(optparse.__file__)[0]
        if not os.path.exists(optparse_source):
            raise RuntimeError("Could not find optparse.py at %s" % optparse_source)
        files.append(optparse_source)
        return files



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

# TODO: test Python3 syntax using flake8
# flake8: noqa

"""
This module contains several utils used for VMs running on clouds
"""

from typing import Union

import logging
import base64

from rhsmlib.facts.host_collector import HostCollector
from rhsmlib.facts.hwprobe import HardwareCollector
from rhsmlib.facts.custom import CustomFactsCollector

from rhsmlib.cloud.detector import CloudDetector
from rhsmlib.cloud.collector import CloudCollector

from rhsmlib.cloud.providers.aws import AWSCloudDetector, AWSCloudCollector
from rhsmlib.cloud.providers.azure import AzureCloudDetector, AzureCloudCollector
from rhsmlib.cloud.providers.gcp import GCPCloudDetector, GCPCloudCollector

# List of detector classes with supported cloud providers
CLOUD_DETECTORS = [
    AWSCloudDetector,
    AzureCloudDetector,
    GCPCloudDetector
]

# List of collector classes with supported cloud provider
CLOUD_COLLECTORS = [
    AWSCloudCollector,
    AzureCloudCollector,
    GCPCloudCollector
]

log = logging.getLogger(__name__)


def detect_cloud_provider() -> list:
    """
    This method tries to detect cloud provider using hardware information provided by dmidecode.
    When there is strong sign that the VM is running on one of the cloud provider, then return
    list containing only one provider. When there is no strong sign of one cloud provider, then
    try to detect cloud provider using heuristics methods. In this case this method will return
    list of all cloud providers sorted according detected probability
    :return: List of string representing detected cloud providers. E.g. ['aws'] or ['aws', 'gcp']
    """

    # Gather only information about hardware, virtualization and custom facts
    facts = {}
    facts.update(HostCollector().get_all())
    facts.update(HardwareCollector().get_all())
    # When cloud provider will change information provided by SM BIOS, then
    # customers will be able to create simple workaround using custom facts.
    facts.update(CustomFactsCollector().get_all())

    cloud_detectors = [cls(facts) for cls in CLOUD_DETECTORS]

    log.debug('Trying to detect cloud provider')

    # First try to detect cloud providers using strong signs
    cloud_list = []
    cloud_detector: CloudDetector
    for cloud_detector in cloud_detectors:
        cloud_detected = cloud_detector.is_running_on_cloud()
        if cloud_detected is True:
            cloud_list.append(cloud_detector.ID)

    # When only one cloud provider was detected, then return the list with
    # one cloud provider. Print error in other cases and try to detect cloud providers
    # using heuristics methods, because it will be necessary to sort the list according
    # probability
    if len(cloud_list) == 1:
        log.info('Detected one cloud provider using strong signs: {provider}'.format(provider=cloud_list[0]))
        return cloud_list
    elif len(cloud_list) == 0:
        log.error('No cloud provider detected using strong signs')
    elif len(cloud_list) > 1:
        log.error('More than one cloud provider detected using strong signs ({providers})'.format(
            providers=", ".join(cloud_list)
        ))

    # When no cloud provider detected using strong signs, because behavior of cloud providers
    # has changed, then try to detect cloud provider using some heuristics
    cloud_list = []
    cloud_detector: CloudDetector
    for cloud_detector in cloud_detectors:
        probability: float = cloud_detector.is_likely_running_on_cloud()
        if probability > 0.0:
            cloud_list.append((probability, cloud_detector.ID))
    # Sort list according probability (provider with highest probability first)
    cloud_list.sort(reverse=True)
    # We care only about order, not probability in the result (filter probability out)
    cloud_list = [item[1] for item in cloud_list]

    if len(cloud_list) == 0:
        log.error('No cloud provider detected using heuristics')
    else:
        log.info('Following list of cloud providers detected using heuristic: {providers}'.format(
            providers=", ".join(cloud_list)
        ))

    return cloud_list


def collect_cloud_info(cloud_list: list) -> dict:
    """
    Try to collect cloud information: metadata and signature provided by cloud provider.
    :param cloud_list: The list of detected cloud providers. In most cases the list contains only one item.
    :return: The dictionary with metadata and signature (when signature is provided by cloud provider).
        Empty dictionary is returned, when it wasn't possible to collect any metadata
    """

    # Create dispatcher dictionary from the list of supported cloud collectors
    cloud_collectors = {
        collector_cls.CLOUD_PROVIDER_ID: collector_cls for collector_cls in CLOUD_COLLECTORS
    }

    result = {}
    # Go through the list of detected cloud providers and try to collect
    # metadata. When metadata are gathered, then break the loop
    for cloud_collector_id in cloud_list:
        cloud_collector: CloudCollector = cloud_collectors[cloud_collector_id]()

        # Try to get metadata first
        metadata: Union[str, None] = cloud_collector.get_metadata()

        # When it wasn't possible to get metadata for this cloud provider, then
        # continue with next detected cloud provider
        if metadata is None:
            log.warning(f'No metadata gathered for cloud provider: {cloud_collector_id}')
            continue

        # Try to get signature
        signature: Union[str, None] = cloud_collector.get_signature()

        # When it is not possible to get signature for given cloud provider,
        # then silently set signature to empty string, because some cloud
        # providers does not provide signatures
        if signature is None:
            signature = ""

        log.info(f'Metadata and signature gathered for cloud provider: {cloud_collector_id}')

        # Encode metadata and signature using base64 encoding. Because base64.b64encode
        # returns values as bytes, then we decode it to string using ASCII encoding.
        b64_metadata: str = base64.b64encode(bytes(metadata, 'utf-8')).decode('ascii')
        b64_signature: str = base64.b64encode(bytes(signature, 'utf-8')).decode('ascii')

        result = {
            'cloud_id': cloud_collector_id,
            'metadata': b64_metadata,
            'signature': b64_signature
        }
        break

    return result


# Some temporary smoke testing code. You can test this module using:
# sudo PYTHONPATH=./src:./syspurse/src python3 -m rhsmlib.cloud.utils
if __name__ == '__main__':
    _result = detect_cloud_provider()
    print('>>> debug <<< result: %s' % _result)



from __future__ import print_function, division, absolute_import

# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import mock

from rhsmlib.facts import host_collector


class HostCollectorTest(unittest.TestCase):

    @mock.patch('locale.getdefaultlocale')
    def test_unknown_locale(self, mock_locale):
        collector = host_collector.HostCollector()
        mock_locale.return_value = (None, None)
        facts = collector.get_all()

        self.assertTrue(isinstance(facts, dict))
        self.assertEqual(facts['system.default_locale'], 'Unknown')

    @mock.patch('locale.getdefaultlocale')
    def test_en_us_utf8_locale(self, mock_locale):
        collector = host_collector.HostCollector()
        mock_locale.return_value = ('en_US', 'UTF-8')
        facts = collector.get_all()

        self.assertTrue(isinstance(facts, dict))
        self.assertEqual(facts['system.default_locale'], 'en_US.UTF-8')

    @mock.patch('locale.getdefaultlocale')
    def test_en_us_no_encoding_locale(self, mock_locale):
        collector = host_collector.HostCollector()
        mock_locale.return_value = ('en_US', None)
        facts = collector.get_all()

        self.assertTrue(isinstance(facts, dict))
        self.assertEqual(facts['system.default_locale'], 'en_US')
